% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [A,Theta,J]=birs_svar(p,h,y,long_restr)
% This function estimates the Impulse Responses and computes various
% bootstrap Confidence Intervals. Procedure follows Ltkepohl (2005) D.3,
% p.709.

% Inputs
% p: number of lags
% h: IR horizon
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables
    % T: number of observations not taken as given
% long_restr: matrix with long-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere

% Outputs
% A - (K*pxK*p) matrix, AR matrix of VAR(1) respresentation of VAR(p)
% Theta - matrix of IRs
% J - matrix to construct IRs

Y=y(:,p+1:end);
[K,T]=size(Y);
[Ahat,U,C]=Mols(y,p);
[Phi,Psi,BB]=svar(p,T,K,Ahat,U,long_restr);


%  --------------- To compute IRs with actual data: -----------------------
[A,Theta,J]=Irs(Ahat,BB,K,p,h);

% Centered residuals, (only if there is no intercept term)?? used in Jmulti
Ubar = sum(U,2)/T;
for t=1:T
    U(:,t) = U(:,t)-Ubar;
end

%% The bootstrap loop
brep = 2000; % No. of bootstrap replications

Bootstrap = zeros(K^2,(h+1),brep);
for n=1:brep
    n %#ok<NOPRT>

    %% Standard bootstrap series (uncomment to use)
    
%     % Draw residuals at random with replacement:
%     Ustar = U(:,ceil(size(U,2)*rand(1,T)));
% 
%     % Set initial values:
%     y_star = zeros(K,T+p);
%     y_star(:,1:p) = y(:,1:p); % Pre-sample values   
%     % Generate normal bootstraped series
%     for t=p+1:T+p
%         for i=1:p
%         y_star(:,t) = y_star(:,t) + Ahat(:,1+K*(i-1):K*i)*y_star(:,t-i);
%         end
%         y_star(:,t) = y_star(:,t) + C + Ustar(:,t-p); % t-p since Ustar starts from 1 and t starts from p+1
%     end
%     
%     % Obtain parameters based on bootstrapped series
%     [Ahat1,U1]=Mols(y_star,p);
%     [Phi1,Psi1,B1]=svar(p,T,K,Ahat1,U1,long_restr);
%     for i = 1:K
%         if B1(i,i) < 0
%             B1(:,i) = -1*B1(:,i);
%         end
%     end
%     [A1,Theta1,J1]=Irs(Ahat1,B1,K,p,h);
    
    
    %% Fixed design wild bootstrap series (uncomment to use)
    
    % Draw residuals at random with replacement:
    Ustar = U(:,ceil(size(U,2)*rand(1,T)));

    % Rademacher distribution:
    rade = 2*round(rand(T,1))-1;
    
    % In case of Wild CI use the Rademacher distribution
    for t=1:T
        Ustar(:,t) = Ustar(:,t)*rade(t,:);
    end

    % Generate wild bootstraped series
    y_star = zeros(K,T+p);
    y_star(:,1:p) = y(:,1:p); % Pre-sample values
    for t=p+1:T+p
        for i=1:p
        y_star(:,t) = y_star(:,t) + Ahat(:,1+K*(i-1):K*i)*y(:,t-i);
        end
        y_star(:,t) = y_star(:,t) + C + Ustar(:,t-p);
    end

    % Obtain parameters based on bootstrapped series    
    Y=y_star(:,p+1:end);
    Z=ones(K*p+1,T);
    for i=1:p
        Z(K*(i-1)+2:K*i+1,:)=y(:,p+1-i:T+p-i); % need the real sample to get sensible results
    end
    Ahat1=Y*Z'/(Z*Z'); % Multivariate OLS
    U1=Y-Ahat1*Z;
    Ahat1=Ahat1(:,2:end);

    [Phi1,Psi1,B1]=svar(p,T,K,Ahat1,U1,long_restr);
    for i = 1:K
        if B1(i,i) < 0
            B1(:,i) = -1*B1(:,i);
        end
    end
    [A1,Theta1]=Irs(Ahat1,B1,K,p,h);

    %% Store the bootstrapped IR values
    
    % Store values in matrix, uncomment for Hall's percentile interval
    Bootstrap(:,:,n) = Theta1;%-Theta;
end

%% Compute Confidence Intervals, uncomment appropriate one:

% Standard deviation Confidence Intervals:
% stdir = std(Bootstrap,1,3);
% CIL2 = Theta-2*stdir;
% CIH2 = Theta+2*stdir;
% CIL1 = Theta-stdir;
% CIH1 = Theta+stdir;

% Hall's percentile interval:
% quantH2 = quantile(Bootstrap,0.975,3);
% quantL2 = quantile(Bootstrap,0.025,3);
% quantH1 = quantile(Bootstrap,0.835,3);
% quantL1 = quantile(Bootstrap,0.165,3);
% CIH2 = Theta-quantL2; % As done in Jmulti
% CIL2 = Theta-quantH2;
% CIH1 = Theta-quantL1;
% CIL1 = Theta-quantH1;

% Efron Confidence Intervals:
CIH2 = quantile(Bootstrap,0.975,3);
CIH1 = quantile(Bootstrap,0.84,3);
CIL2 = quantile(Bootstrap,0.025,3);
CIL1 = quantile(Bootstrap,0.16,3);

%% Plot of Impulse Responses with Confidence Intervals
Theta = reshape(Theta,K,K,h+1);
CIH2 = reshape(CIH2,K,K,h+1); CIH1 = reshape(CIH1,K,K,h+1);
CIL2 = reshape(CIL2,K,K,h+1); CIL1 = reshape(CIL1,K,K,h+1);
figure('Name','Impulse Responses','NumberTitle','off')
periods=0:h;

subplot(3,3,1)
plot(periods,reshape(Theta(1,1,:),1,h+1),'b-',periods,reshape(CIH2(1,1,:),1,h+1),'r-.',periods,reshape(CIH1(1,1,:),1,h+1),'g-.',periods,reshape(CIL1(1,1,:),1,h+1),'g-.',periods,reshape(CIL2(1,1,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
title('Fundamental Shock 1')
ylabel('Real GDP')

subplot(3,3,2)
plot(periods,reshape(Theta(1,2,:),1,h+1),'b-',periods,reshape(CIH2(1,2,:),1,h+1),'r-.',periods,reshape(CIH1(1,2,:),1,h+1),'g-.',periods,reshape(CIL1(1,2,:),1,h+1),'g-.',periods,reshape(CIL2(1,2,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
title('Fundamental Shock 2')

subplot(3,3,3)
plot(periods,reshape(Theta(1,3,:),1,h+1),'b-',periods,reshape(CIH2(1,3,:),1,h+1),'r-.',periods,reshape(CIH1(1,3,:),1,h+1),'g-.',periods,reshape(CIL1(1,3,:),1,h+1),'g-.',periods,reshape(CIL2(1,3,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
title('Non-fundamental Shock')

subplot(3,3,4)
plot(periods,reshape(Theta(2,1,:),1,h+1),'b-',periods,reshape(CIH2(2,1,:),1,h+1),'r-.',periods,reshape(CIH1(2,1,:),1,h+1),'g-.',periods,reshape(CIL1(2,1,:),1,h+1),'g-.',periods,reshape(CIL2(2,1,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
ylabel('Real Interest Rates')

subplot(3,3,5)
plot(periods,reshape(Theta(2,2,:),1,h+1),'b-',periods,reshape(CIH2(2,2,:),1,h+1),'r-.',periods,reshape(CIH1(2,2,:),1,h+1),'g-.',periods,reshape(CIL1(2,2,:),1,h+1),'g-.',periods,reshape(CIL2(2,2,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

subplot(3,3,6)
plot(periods,reshape(Theta(2,3,:),1,h+1),'b-',periods,reshape(CIH2(2,3,:),1,h+1),'r-.',periods,reshape(CIH1(2,3,:),1,h+1),'g-.',periods,reshape(CIL1(2,3,:),1,h+1),'g-.',periods,reshape(CIL2(2,3,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

subplot(3,3,7)
plot(periods,reshape(Theta(3,1,:),1,h+1),'b-',periods,reshape(CIH2(3,1,:),1,h+1),'r-.',periods,reshape(CIH1(3,1,:),1,h+1),'g-.',periods,reshape(CIL1(3,1,:),1,h+1),'g-.',periods,reshape(CIL2(3,1,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
ylabel('Real Stock Prices')

subplot(3,3,8)
plot(periods,reshape(Theta(3,2,:),1,h+1),'b-',periods,reshape(CIH2(3,2,:),1,h+1),'r-.',periods,reshape(CIH1(3,2,:),1,h+1),'g-.',periods,reshape(CIL1(3,2,:),1,h+1),'g-.',periods,reshape(CIL2(3,2,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

subplot(3,3,9)
plot(periods,reshape(Theta(3,3,:),1,h+1),'b-',periods,reshape(CIH2(3,3,:),1,h+1),'r-.',periods,reshape(CIH1(3,3,:),1,h+1),'g-.',periods,reshape(CIL1(3,3,:),1,h+1),'g-.',periods,reshape(CIL2(3,3,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

end

function [Ahat,U,C]=Mols(y,p)

Y=y(:,p+1:end);
[K,T]=size(Y);
Z=ones(K*p+1,T);
for i=1:p
    Z(K*(i-1)+2:K*i+1,:)=y(:,p+1-i:T+p-i);
end
Ahat=Y*Z'/(Z*Z'); % Multivariate OLS 3.2.10, p.72 in Ltkepohl(2005)
U=Y-Ahat*Z;
C=Ahat(:,1);
Ahat=Ahat(:,2:end);

end

function [A,Theta,J]=Irs(Ahat,B,K,p,h)
A=zeros(K*p,K*p); % Define the A matrix
A(1:K,1:end)=Ahat;
for m=1:p-1
    A(m*K+1:m*K+K,K*m-K+1:K*m)=eye(K);
end
J=[eye(K,K) zeros(K,K*(p-1))]; % Define the J matrix
Theta=reshape(J*A^0*J'*B,K^2,1); % Impulse response matrix
for i=1:h
    IR = reshape(J*A^0*J'*B,K^2,1);
    for j = 1:i
        IR = IR + reshape(J*A^j*J'*B,K^2,1);
    end
    Theta=([Theta IR]);
end

end