% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [A,Theta,J]=birs_svecm(p,r,h,y,long_restr,short_restr,fixed,trend)
% This function estimates the impulse responses (IRs) and computes various
% bootstrap confidence intervals. Procedure follows Ltkepohl (2005) D.3,
% p.709.

% Inputs
% r: cointegration rank
% p: number of lags as in the VAR form of the VECM
% h: IR horizon
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables
    % T: number of observations not taken as given
% long_restr: matrix with long-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere
% short_restr: same as long_restr for restrictions directly on B
% fixed: starting values of the optimization algorithm estimating the restricted parameters, if fixed = 'y' starting values are 0.1, if = 'n' then starting values are random numbers
% trend: if trend = 'y', D_t = nu1 + nu2*t, otherwise D_t = nu1.

% Outputs
% A - (K*pxK*p) matrix, AR matrix of VAR(1) respresentation of VAR(p)
% Theta - matrix of IRs
% J - matrix to construct IRs

[K,T,alpha,beta,C,~,Gamma,~,BB,~,~,~,~,U] = svecm(y,p,r,long_restr,short_restr,fixed,trend);
Ahat = vectovar(alpha,beta,Gamma,K,p);

%  --------------- To compute IRs with actual data: -----------------------
[A,Theta,J]=Irs(Ahat,BB,K,p,h);
Deltay = diff(y,1,2);
% Centered residuals, (only if there is no intercept term)?? used in Jmulti
Ubar = sum(U,2)/T;
for t=1:T
    U(:,t) = U(:,t)-Ubar;
end

%% The bootstrap loop
brep = 200; % No. of bootstrap replications

Bootstrap = zeros(K^2,(h+1),brep);
for n=1:brep
    n %#ok<NOPRT>

    %% Generate standard bootstrap series (uncomment to use)
    % Draw residuals at random with replacement:
%     Ustar = U(:,ceil(size(U,2)*rand(1,T)));
% 
%     y_star = zeros(K,T+p-1); % Differenced series
%     Yminus1 = zeros(K,T+p); Yminus1(:,1:p-1) = y(:,1:p-1); % Levels series
%     y_star(:,1:p-1) = Deltay(:,1:p-1); % Pre-sample values
%     for t=p:T+p-1
%         for i=1:p-1
%         y_star(:,t) = y_star(:,t) + Gamma(:,1+K*(i-1):K*i)*y_star(:,t-i);
%         end
%         Yminus1(:,t) = Yminus1(:,t-1) + y_star(:,t-1);
%         y_star(:,t) = y_star(:,t) + C + -alpha*beta'*Yminus1(:,t) + Ustar(:,t-p+1); % t-p+1 since Ustar starts from 1 and t starts from p
%     end
% 
%     % Obtain parameters based on bootstrapped series
%     [alpha1,beta1,Gamma1,B1] = get_pars(Yminus1,y_star,p,r,T,K,long_restr,short_restr,fixed);
%     for i = 1:K
%         if B1(i,i) < 0
%             B1(:,i) = -1*B1(:,i);
%         end
%     end
%     Ahat1 = vectovar(alpha1,beta1,Gamma1,K,p);
%     [A1,Theta1]=Irs(Ahat1,B1,K,p,h);

    %% Fixed design wild bootstrap series (uncomment to use)
    % Draw residuals at random with replacement:
    Ustar = U(:,ceil(size(U,2)*rand(1,T)));

    % Rademacher distribution:
    rade = 2*round(rand(T,1))-1;
    
    % In case of Wild CI use the Rademacher distribution, otherwise comment
    for t=1:T
        Ustar(:,t) = Ustar(:,t)*rade(t,:);
    end

    y_star = zeros(K,T+p-1);
    y_star(:,1:p-1) = Deltay(:,1:p-1); % Pre-sample values
    for t=p:T+p-1
        for i=1:p-1
        y_star(:,t) = y_star(:,t) + Gamma(:,1+K*(i-1):K*i)*Deltay(:,t-i);
        end
        y_star(:,t) = y_star(:,t) + C + -alpha*beta'*y(:,t) + Ustar(:,t-p+1); % t-p+1 since Ustar starts from 1 and t starts from p
    end

    % Obtain parameters based on bootstrapped series
    [alpha1,beta1,Gamma1,B1] = get_pars(y,y_star,p,r,T,K,long_restr,short_restr,fixed);
%     for i = 1:K
%         if B1(i,i) < 0
%             B1(:,i) = -1*B1(:,i);
%         end
%     end
    Ahat1 = vectovar(alpha1,beta1,Gamma1,K,p);
    [A1,Theta1]=Irs(Ahat1,B1,K,p,h);
    
    %% Store the bootstrapped IR values
    
    % Store values in matrix, uncomment for Hall's percentile interval
    Bootstrap(:,:,n) = Theta1;%-Theta;
end

%% Compute Confidence Intervals, uncomment appropriate one:

% Standard deviation Confidence Intervals:
% stdir = std(Bootstrap,1,3);
% CIL2 = Theta-2*stdir;
% CIH2 = Theta+2*stdir;
% CIL1 = Theta-stdir;
% CIH1 = Theta+stdir;

% Hall's percentile interval:
% quantH2 = quantile(Bootstrap,0.975,3);
% quantL2 = quantile(Bootstrap,0.025,3);
% quantH1 = quantile(Bootstrap,0.835,3);
% quantL1 = quantile(Bootstrap,0.165,3);
% CIH2 = Theta-quantL2; % As done in Jmulti
% CIL2 = Theta-quantH2;
% CIH1 = Theta-quantL1;
% CIL1 = Theta-quantH1;

% Efron Confidence Intervals:
CIH2 = quantile(Bootstrap,0.975,3);
CIH1 = quantile(Bootstrap,0.84,3);
CIL2 = quantile(Bootstrap,0.025,3);
CIL1 = quantile(Bootstrap,0.16,3);

%% Plot of Impulse Responses with Confidence Intervals
Theta = reshape(Theta,K,K,h+1);
CIH2 = reshape(CIH2,K,K,h+1); CIH1 = reshape(CIH1,K,K,h+1);
CIL2 = reshape(CIL2,K,K,h+1); CIL1 = reshape(CIL1,K,K,h+1);
figure('Name','Impulse Responses','NumberTitle','off')
periods=0:h;

subplot(3,3,1)
plot(periods,reshape(Theta(1,1,:),1,h+1),'b-',periods,reshape(CIH2(1,1,:),1,h+1),'r-.',periods,reshape(CIH1(1,1,:),1,h+1),'g-.',periods,reshape(CIL1(1,1,:),1,h+1),'g-.',periods,reshape(CIL2(1,1,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
title('Fundamental Shock 1')
ylabel('Real Earnings')

subplot(3,3,2)
plot(periods,reshape(Theta(1,2,:),1,h+1),'b-',periods,reshape(CIH2(1,2,:),1,h+1),'r-.',periods,reshape(CIH1(1,2,:),1,h+1),'g-.',periods,reshape(CIL1(1,2,:),1,h+1),'g-.',periods,reshape(CIL2(1,2,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
title('Fundamental Shock 2')

subplot(3,3,3)
plot(periods,reshape(Theta(1,3,:),1,h+1),'b-',periods,reshape(CIH2(1,3,:),1,h+1),'r-.',periods,reshape(CIH1(1,3,:),1,h+1),'g-.',periods,reshape(CIL1(1,3,:),1,h+1),'g-.',periods,reshape(CIL2(1,3,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
title('Non-fundamental Shock')

subplot(3,3,4)
plot(periods,reshape(Theta(2,1,:),1,h+1),'b-',periods,reshape(CIH2(2,1,:),1,h+1),'r-.',periods,reshape(CIH1(2,1,:),1,h+1),'g-.',periods,reshape(CIL1(2,1,:),1,h+1),'g-.',periods,reshape(CIL2(2,1,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
ylabel('Real Interest Rates')

subplot(3,3,5)
plot(periods,reshape(Theta(2,2,:),1,h+1),'b-',periods,reshape(CIH2(2,2,:),1,h+1),'r-.',periods,reshape(CIH1(2,2,:),1,h+1),'g-.',periods,reshape(CIL1(2,2,:),1,h+1),'g-.',periods,reshape(CIL2(2,2,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

subplot(3,3,6)
plot(periods,reshape(Theta(2,3,:),1,h+1),'b-',periods,reshape(CIH2(2,3,:),1,h+1),'r-.',periods,reshape(CIH1(2,3,:),1,h+1),'g-.',periods,reshape(CIL1(2,3,:),1,h+1),'g-.',periods,reshape(CIL2(2,3,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

subplot(3,3,7)
plot(periods,reshape(Theta(3,1,:),1,h+1),'b-',periods,reshape(CIH2(3,1,:),1,h+1),'r-.',periods,reshape(CIH1(3,1,:),1,h+1),'g-.',periods,reshape(CIL1(3,1,:),1,h+1),'g-.',periods,reshape(CIL2(3,1,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;
ylabel('Real Stock Prices')

subplot(3,3,8)
plot(periods,reshape(Theta(3,2,:),1,h+1),'b-',periods,reshape(CIH2(3,2,:),1,h+1),'r-.',periods,reshape(CIH1(3,2,:),1,h+1),'g-.',periods,reshape(CIL1(3,2,:),1,h+1),'g-.',periods,reshape(CIL2(3,2,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

subplot(3,3,9)
plot(periods,reshape(Theta(3,3,:),1,h+1),'b-',periods,reshape(CIH2(3,3,:),1,h+1),'r-.',periods,reshape(CIH1(3,3,:),1,h+1),'g-.',periods,reshape(CIL1(3,3,:),1,h+1),'g-.',periods,reshape(CIL2(3,3,:),1,h+1),'r-.',periods,zeros(size(periods)),'k-','LineWidth',2); axis tight;

end

function [A,Theta,J]=Irs(Ahat,B,K,p,h)
A=zeros(K*p,K*p); % Define the A matrix
A(1:K,1:end)=Ahat;
for m=1:p-1
    A(m*K+1:m*K+K,K*m-K+1:K*m)=eye(K);
end
J=[eye(K,K) zeros(K,K*(p-1))]; % Define the J matrix
Theta=reshape(J*A^0*J'*B,K^2,1); % Impulse response matrix
for i=1:h
    Theta=([Theta reshape(J*A^i*J'*B,K^2,1)]);
end

end

function Ahat = vectovar(alpha,beta,Gamma,K,p)
% Converts VEC to VAR parameters as in Ltkepohl (2005)

Ahat = zeros(K,K*p);
Ahat(:,1:K) = -alpha*beta' + eye(K) + Gamma(:,1:K);
for i = 2:p-1
    Ahat(:,K*(i-1)+1:K*i) = Gamma(:,K*(i-1)+1:K*i) - Gamma(:,K*(i-2)+1:K*(i-1));
end
Ahat(:,K*(p-1)+1:K*p) = -Gamma(:,K*(p-2)+1:K*(p-1));
end

function [alpha,beta,Gamma,B] = get_pars(y,y_star,p,r,T,K,long_restr,short_restr,fixed)
%% Standard bootstrap (uncomment to use)
% Deltay = y_star;
% Yminus1 = y;

%% Fixed design wild bootstrap (uncomment to use)
Deltay = diff(y,1,2); % need the real sample to get sensible results
Yminus1 = zeros(K,T+p); Yminus1(:,1) = y(:,1);
for t = 2:T+p
    Yminus1(:,t) = Yminus1(:,t-1) + y_star(:,t-1);
end

%% Main code starts here
Yminus1 = Yminus1(:,p:end-1);
DeltaY = y_star(:,p:end);

Z = ones(K*(p-1)+1,T);
for i = 1:p-1
    Z(K*(i-1)+2:K*i+1,:) = Deltay(:,p-i:end-i);
end

Pihat = DeltaY*Z'/(Z*Z'); % replaced inv(Z*Z') with /(Z*Z')
Uhat = DeltaY-Pihat*Z;
Nhat = Yminus1*Z'/(Z*Z');
Vhat = Yminus1-Nhat*Z;
SIGMAvv = Vhat*Vhat'/T;
SIGMAuu = Uhat*Uhat'/T;
SIGMAuv = Uhat*Vhat'/T;
SIGMAvu = SIGMAuv';

[a lambda] = eigs(SIGMAvv\SIGMAvu/(SIGMAuu)*SIGMAuv,r); %#ok<NASGU>

beta_b = zeros(size(a));
for i = 1:size(a,2)
    beta_b(:,i) = a(:,i)/(a(:,i)'*SIGMAvv*a(:,i))^0.5; % [20.2.11]
end
beta_r = beta_b(1:r,:); % Normalization as in Ltkepohl (2005)
beta = beta_b/(beta_r); % the beta matrix
alpha = -SIGMAuv*beta_b*beta_r'; % the alpha matrix
Pi = -alpha*beta'; % estimated Pi matrix

for i = 1:p-1
    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+2:K*i+1) - Pi*Nhat(:,K*(i-1)+2:K*i+1);
end
nu1 = Pihat(:,1) - Pi*Nhat(:,1);
Gam = [nu1 Gamma];
U1 = DeltaY-Pi*Yminus1-Gam*Z;
S_uhat = U1*U1'/T;

beta_orth = null(beta','r');
alpha_orth = null(alpha','r');
lagpolynomial = zeros(K,K);
for i = 1:p-1
    lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
end
lagpolynomial = eye(K) + lagpolynomial;
Xi_tilde = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';

% Restrictions on the B matrix specified as in (9.2.3), p.370 of Ltkepohl (2005)
if size(long_restr,1) == 0
    C = short_restr;
elseif size(short_restr,1) == 0
    long_restr = long_restr*kron(eye(K),Xi_tilde);
    C = long_restr;
else
    long_restr = long_restr*kron(eye(K),Xi_tilde);
    C = [long_restr; short_restr];
end

% Optimization algorithm according to Amisano & Giannini (1997)
% Define the restrictions
R = null(C,'r'); % in the format of (9.1.12), picks up the unrestricted elements

% Set starting values
if fixed == 'y'
    start_val = ones(size(R,2),1)*0.1;
else
    start_val = randn(size(R,2),1);
end

vecB = R*start_val;

Com = zeros(K^2,K);
c = 1;
r = 1;
while c <= K
    Com(r,c) = 1;
    c = c + 1;
    r = r + K;
end
zold = Com;
cc = 1;
while cc < K
    z = [zeros(cc,K); zold(1:K^2-cc,:)];
    Com = [Com z]; %#ok<AGROW>
    cc = cc + 1;
end
B = reshape(vecB,K,K);

% Using (9.3.4) and (9.3.8) in (9.3.7)
i = 0;
while i <= 500
    
    I = T*R'*-(kron(eye(K),inv(B')))*(eye(K^2) + Com)*(-(kron(eye(K),inv(B))))*R;
    score = R'*-(kron(inv(B),inv(B')))*T*(reshape(B',K^2,1) - kron(S_uhat,eye(K))*reshape(inv(B),K^2,1));
    
    length = max(abs(I\score));
    if length > 1
        step = 1/length;
    else
        step = 1;
    end
    
    start_val = start_val + I\score*step;
    vecB = R*start_val;
    B = reshape(vecB,K,K);
    
    i = i + 1;
end

% Look at positive shocks only
for i = 1:K
    if B(i,i) < 0
        B(:,i) = -1*B(:,i);
    end
end
end