% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [loglikelihood,xiT,P2] = expectation_mssvecm(T,K,M,Sigma,U,xi1,P1,intercept)
%% Expectation Step of the EM algorithm

%% Conditional Densities
% Matrix of conditional densities, eta, for t = 1,...,T
eta = zeros(M,T);

invSigma = zeros(K,K,M);
sqdetSigma = zeros(M,1);
for m = 1:M
    invSigma(:,:,m) = inv(Sigma(:,:,m));
    sqdetSigma(m,:) = sqrt(det(Sigma(:,:,m)));
end

if intercept == 'n'
    for t=1:T
        for m = 1:M
            eta(m,t) = exp(-U(:,t)'*invSigma(:,:,m)*U(:,t)/2)/((2*pi)^(K/2)*sqdetSigma(m,:));
        end
    end
else
    for t=1:T
        for m = 1:M
            eta(m,t) = exp(-U(:,t,m)'*invSigma(:,:,m)*U(:,t,m)/2)/((2*pi)^(K/2)*sqdetSigma(m,:));
        end
    end
end

%% Filtered Probabilities
% Matrix of filtered probabilities, xit on the basis of the parameters from
% the previous iteration. Note that time period 0 is t=1 and time period 1 is t=2,
% since matlab does not allow for a 0 index in a matrix.
% Probabilities are calculated for t = 1,...,T with xit_0|0 given
xit = zeros(M,T+1);
xit(:,1) = xi1;
loglikelihood = zeros(1,T); % Calculate the log likelihood.

for t=2:T+1
    j=t-1;
    xit(:,t) = ((P1'*xit(:,t-1)).*eta(:,j))/(ones(1,M)*((P1'*xit(:,t-1)).*eta(:,j))); % Krolzig (1997) (5.4) p.82
    loglikelihood(:,t) = log(ones(1,M)*((P1'*xit(:,t-1)).*eta(:,j)));
end

loglikelihood = sum(loglikelihood,2);

%% Smoothed Probabilities
% Matrix of smoothed probabilities, xiT on the basis of the filtered
% probabilities, for t = T-1,...,0 with xiT_T|T given
xiT = zeros(M,T+1);

xiT(:,end) = xit(:,end);
% Optional possibility of avoiding label switching
%xiT(:,end) = sort(xit(:,end));

for t=1:T
    xiT(:,end-t) = (P1*(xiT(:,end-t+1)./(P1'*xit(:,end-t)))).*xit(:,end-t); % Krolzig (1997) (5.13) p.85
end

%% The joint distribution of s_t+1 and s_t
% xihat2 stores the joint probabilities of P(s_t+1,s_t|Y_T) for
% t = 0,...,T-1
xihat2 = zeros(M^2,T);

for t=1:T
    xihat2(:,t) = reshape(P1,M^2,1).*(kron((xiT(:,t+1)./(P1'*xit(:,t))),xit(:,t))); % Krolzig (1997) p.90
end

xihat2 = sum(xihat2,2);

% Needed to estimate the transition probabilities
xihat1 = kron(ones(1,M),eye(M))*xihat2; % equivalent formulation: xihat1 = sum(xiT(:,1:T-1),2);

%% Transition Probabilities - Hidden Markov Chain Step
% Estimation based on the previous transition matrix, filtered,
% smoothed and joint probabilites

rho = xihat2./(kron(ones(M,1),xihat1));
P2 = reshape(rho,M,M);