% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [LRtest,Pars_lr] = lrtestsvar(p,M,y,K,loglike_ms,short_restr,long_restr,intercept,slope)
% This function gives the Likelihood Ratio test reults for joint equality
% of the Lambda matrices parameters in the MS-SVAR model. It is generalized to states and
% number of variables.
% The LR tests have the form of lambda_11,1 = lambda_22,1, ... , lambda_11,M = lambda_22,M,
% and then lambda_11,1 = lambda_33,1, ... , lambda_11,M = lambda_33,M, etc.

% INPUTS:
% p - number of lags
% M - number of Markov states
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables
    % T: number of observations not taken as given
% loglike_ms - log-likelihood of the Markov switching (MS) SVAR model
% short_restr: matrix with short-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere
% long_restr - same as short_restr for restrictions on A(1)^-1*B
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise.
% slope: slope = 'y' for a switching AR matrices, slope = 'n' otherwise.

% OUTPUTS:
% Lrtest - (Kx(K-1)/2x3) matrix of log-likelihood, LR test and p-values.
% Pars_lr - ((K^2 + K*(M-1))x(K-1)/2) matrix of covariance decomposition parameters, B and Lambda_i, i=2,...,M.

%% Define the Restriction matrix
A = [ones(K-1,1) -eye(K-1,K-1)];
BB = A;

i = 2;
while i < K
    Z = [zeros(K-i,i-1) BB(1:K-i,1:K-i+1)];
    A = [A; Z]; %#ok<AGROW>
    i = i + 1;
end

Restrictions = zeros((M-1)*K*(K-1)/2,K*(M-1));
for m = 1:M-1
    Restrictions(K*(K-1)/2*(m-1)+1:K*(K-1)/2*m,K*(m-1)+1:K*m) = A;
end
Restrictions = [zeros((M-1)*K*(K-1)/2,K^2) Restrictions];

% This matrix is used for the fmincon function and is used in particular for
% the nonlinear equality constraint in nonlcon such that Ceq(pars) = 0.
% In our case it is Ceq*pars = 0, where pars are the Lambda matrix
% parameters.
Ceq = zeros(M-1,size(Restrictions,2),K*(K-1)/2);
j = 1;
for m =1:M-1
    for i = 1:size(Ceq,3)
        Ceq(m,:,i) = Restrictions(j,:);
        j = j + 1;
    end
end

% This matrix ensures that the same optimum is not reached at different
% tests. It comes as an inequality constraint in nonlcon such that C*pars
% <= 0.
if K <= 2
    C = [];
else
    C = zeros(size(Ceq));
    h = 1;
    k = K - 1;
    j = 1;
    while h <= size(Ceq,3)
        i = 1;
        while i <= k
            if h ~= j
                C(:,:,h) = Ceq(:,:,j);
            elseif h == j
                if j == size(Ceq,3)
                    C(:,:,h) = Ceq(:,:,j-1);
                else
                    C(:,:,h) = Ceq(:,:,j+1);
                end
            end
            h = h + 1;
            i = i + 1;
        end
        j = j + k;
        k = k - 1;
    end
end

%% Obtain the LR test results

df = M-1;
Pars_lr = zeros(K^2 + K*(M-1),1);
LRtest = zeros(size(Ceq,3),2);
Llike = zeros(size(Ceq,3),1);
for i = 1:size(Ceq,3)
    ceq = @(x) Ceq(:,:,i)*x;
    c = @(x) C(:,:,i)*x;
    [llike_restr,pars] = llike(p,M,y,short_restr,long_restr,intercept,slope,ceq,c);
    Llike(i) = llike_restr;
    LRtest(i,1) = 2*(loglike_ms - Llike(i));
    LRtest(i,2) = 1 - chi2cdf(LRtest(i,1),df);
    Pars_lr(:,i) = pars;
end
LRtest = [Llike LRtest];

display('The restricted log-likelihood value, the Likelihood Ratio test and p value are')
display(LRtest)

end

function [llike_restr,pars] = llike(p,M,y,short_restr,long_restr,intercept,slope,ceq,c)

%% Set up exogenous matrices

% Define the exogenous matrices to be used throughout:
[Y,K,T,Zbar,Xbar,bold_y,z] = initialization(y,p,M,intercept,slope);

%% Initialize storing matrices

storeloglike = zeros(1,1);
Parameters = zeros(K^2 + K*(M-1),1);

for o = 1:100
    o %#ok<NOPRT>
    
    %% Initial Values
    % beta is a vector of intercept and vectorized slope parameters
    if slope == 'n'
        if intercept == 'n' % MSH(M)-SVAR(p) Model
            beta = kron(Xbar'*Xbar,eye(K))\kron(Xbar',eye(K))*bold_y; % replaced inv(A)*b by A\b
            U = bold_y - kron(Xbar,eye(K))*beta;
            U = reshape(U,K,T);
            B = sqrtm(U*U'/T) + randn(K,K)/100; % Starting value for the B matrix
            Apars = reshape(beta(K+1:end),K,K*p);
        else % MSIH(M)-SVAR(p) Model
            XX = Xbar(:,M:end,M);

            beta = (XX'*XX)\XX'*Y';
            
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = Y - beta'*XX';
            end

            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
            Apars = beta(2:end,:)';
        end
    else
        if intercept == 'n' % MSAH(M)-SVAR(p) Model
            XX = Xbar(:,1:1+K*p,1);
            
            beta = (XX'*XX)\XX'*Y';
            
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = Y - beta'*XX';
            end
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
            Apars = beta(2:end,:)';
        else % MSIAH(M)-SVAR(p) Model
            Beta = zeros(K*(K*p+1),M);
            U = zeros(K*T,M);
            for m = 1:M
                Beta(:,m) = kron(Xbar'*Xbar,eye(K))\kron(Xbar',eye(K))*bold_y;
                U(:,m) = bold_y-kron(Xbar,eye(K))*Beta(:,m);
            end
            U = reshape(U,K,T,M);
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
            Apars = reshape(Beta(K+1:end,1),K,K*p);
        end
    end
    
    lagpolynomial = zeros(K,K);
    for i = 1:K:K*p
        lagpolynomial = lagpolynomial - Apars(:,i:i+K-1);
    end
    lagpolynomial = eye(K)+lagpolynomial; % lag polynomial, A(1) = I_K - A_1 - A_2 - ... - A_p.
    invA1 = inv(lagpolynomial); % Inverse of lag polynomial, A(1)^-1.
    
    Lambda = zeros(K,K,M);
    Lambda(:,:,1) = eye(K); % Starting value for the Lambda matrix
    for m = 2:M
        Lambda(:,:,m) = o^(m-1)*eye(K);
    end
    
    Sigma = zeros(K,K,M);
    for m = 1:M
        Sigma(:,:,m) = B*Lambda(:,:,m)*B';
    end
    
    P1 = ones(M,1)*ones(1,M)/M; % Starting value of the transition probabilities. Note
    % probabilities add up to 1 row-wise.
    xi1 = ones(M,1)/M; % Starting value of the initial xi vector
    
    % Store the log-likelihood values per iteration for convergence criteria.
    Loglikelihood = zeros(1,1);
    
    % Exit condition, to be used in case of singulariites
    exit = 0;
    %% Main Program using the EM algorithm
    n = 1 %#ok<NOPRT>
    conv = 1;
    while conv>10^-6
        %% Expectation Step
        
        [loglikelihood,xiT,P2] = expectation_mssvar(T,K,M,Sigma,U,xi1,P1,intercept,slope);
        
        %% Maximization Step
        
        % Define the variables, these are from the expectation step and stay
        % outside of the maximization loop.
        
        XIhat = zeros(T,T,M);
        for m = 1:M
            XIhat(:,:,m) = diag(xiT(m,2:end));
        end
        
        That = sum(xiT(:,2:T+1),2);        
        
        %% Estimate the Covariance Matrices and the remaining Parameters
        % Here the lambda parameters are restricted according to the
        % restrictions in Ceq and C which are unsed in nonlcon.
        
        % Options displaying results.
        options = optimset('display','final','Algorithm', 'interior-point','MaxFunEvals', 20000, ...
            'MaxIter', 20000, 'Hessian','bfgs','DerivativeCheck','on','Diagnostics','off','GradObj','off','LargeScale','off');
        
        % Lower bound for the Lambda parameters for fmincon
        lb = -inf(K^2 + K*(M-1),1);
        lb(end-K*(M-1)+1:end,1) = 0.001;
        
        j = 1;
        Fval = zeros(1,1); % Matrix storing the values of the log-likelihood function.
        Pars = zeros(K^2 + K*(M-1),1); % Stores the values of the parameter estimates.
        conv2 = 1;
        
        while conv2>10^-6
            
            Lambdapars = zeros(K*(M-1),1);
            for m = 2:M
                Lambdapars(1+K*(m-2):K*(m-1),:) = diag(Lambda(:,:,m));
            end
            
            pars0 = [B(:); Lambdapars];
            
            % Defines the restrictions to be used on the lambda parameters.
            if K <= 2
                nonlcon = @(pars0) deal([], ceq(pars0));
            else
                nonlcon = @(pars0) deal(c(pars0) ,ceq(pars0));
            end
            
            % Set up restrictions and estimate the B and Lambda parameters.
            if size(long_restr,1) + size(short_restr,1) == 0 % unrestricted case
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept,slope), pars0,[],[],[],[],lb,[],nonlcon,options);
            else % Models with restrictions
                if size(long_restr,1) == 0 % only short-run restrictions
                    aeq = short_restr;
                elseif size(short_restr,1) == 0 % only long-run restrictions
                    aeq = long_restr*kron(eye(K),invA1);
                else % long and short-run restrictions
                    long_restr_updated = long_restr*kron(eye(K),invA1);
                    aeq = [long_restr_updated; short_restr];
                end
                Aeq = [aeq zeros(size(aeq,1),K*(M-1))];
                beq = zeros(size(Aeq,1),1);
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept,slope), pars0,[],[],Aeq,beq,lb,[],nonlcon,options);
            end
            
            Fval(j+1) = fval;
            Pars(:,j) = pars;
            
            % Return the parameter estimates.
            Bhat = reshape(pars(1:K^2),K,K);
            Lambdahat = zeros(K,K,M);
            Lambdahat(:,:,1) = eye(K);
            for m = 2:M
                Lambdahat(:,:,m) = diag(pars(K^2+1+K*(m-2):K^2+K*(m-1)));
            end
            
            Sigmahat = zeros(K,K,M);
            for m = 1:M
                Sigmahat(:,:,m) = Bhat*Lambdahat(:,:,m)*Bhat';
            end
            
            % Check the eigenvalues of the covariance matrices. If smaller
            % than a certain number exit and start from the beginning.
            not_number = isnan(Sigmahat);
            infinite = isinf(Sigmahat);
            not_number = sum(not_number(:));
            infinite = sum(infinite(:));
            if not_number + infinite > 0
                exit = 1;
            else
                Eigenvalues_Sigmahat = zeros(K,1,M);
                for m = 1:M
                    Eigenvalues_Sigmahat(:,1,m) = eig(Sigmahat(:,:,m));
                end
                low_eigenvalues = Eigenvalues_Sigmahat < 0.000001;
                low_eigenvalues = sum(low_eigenvalues(:));
                if low_eigenvalues > 0
                    exit = 1;
                end
            end
            
            % Estimate the intercept and slope parameters according to
            % Table 9.12 p.192 of Krolzig(1997)
            A = zeros(z,z);
            AA = zeros(z,K*T);
            if slope == 'n'
                if intercept == 'n' % MSH(M)-SVAR(p) Model
                    for m = 1:M
                        A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    betahat = A\AA*bold_y;
                    U_hat = bold_y-kron(Xbar,eye(K))*betahat;% The residual matrix.
                    U_hat = reshape(U_hat,K,T);
                    Apars = reshape(betahat(K+1:end),K,K*p); % store the estimated slope parameters
                else % MSIH(M)-SVAR(p) Model
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    betahat = A\AA*bold_y;
                    U_hat = zeros(1,K*T,M); % Residual matrices with switching intercept term
                    for m = 1:M
                        U_hat(:,:,m) = bold_y - kron(XX,eye(K))*[betahat(1+K*(m-1):K*m,1); betahat(K*M+1:end,1)];
                    end
                    U_hat = reshape(U_hat,K,T,M);
                    Apars = reshape(betahat(M*K+1:end),K,K*p);
                end
            else % MSAH(M)-SVAR(p) Model
                if intercept == 'n'
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    betahat = A\AA*bold_y;
                    U_hat = zeros(K,T,M);
                    for m = 1:M
                        U_hat(:,:,m) = Y - reshape(betahat(1+K+K^2*p*(m-1):K+K^2*p*m),K,K*p)*Zbar' - kron(ones(T,1)',betahat(1:K));
                    end
                    A1 = 0; AA1 = A1;
                    for m = 1:M
                        A1 = A1 + kron(XX'*XIhat(:,:,m)*XX,inv(Sigmahat(:,:,m)));
                        AA1 = AA1 + kron(XX'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    Apars = A1\AA1*bold_y;
                    Apars = reshape(Apars(K+1:end),K,K*p);
                else % MSIAH(M)-SVAR(p) Model
                    Betahat = zeros(z/M,M);
                    for m = 1:M
                        Betahat(:,m) = kron((Xbar'*XIhat(:,:,m)*Xbar)\Xbar'*XIhat(:,:,m),eye(K))*bold_y;
                    end
                    U_hat = zeros(1,K*T,M);
                    for m = 1:M
                        U_hat(:,:,m) = bold_y-kron(Xbar,eye(K))*Betahat(:,m);
                    end
                    U_hat = reshape(U_hat,K,T,M);
                    A1 = 0; AA1 = A1;
                    for m = 1:M
                        A1 = A1 + Xbar'*XIhat(:,:,m)*Xbar;
                        AA1 = AA1 + Xbar'*XIhat(:,:,m);
                    end
                    Apars = kron(A1\AA1,eye(K))*bold_y;
                    Apars = reshape(Apars(K+1:end),K,K*p);
                end
            end
            
            if size(long_restr,1) > 0
                % Get the new value of the lagpolynomial.
                lagpolynomial = zeros(K,K);
                for i = 1:K:K*p
                    lagpolynomial = lagpolynomial - Apars(:,i:i+K-1);
                end
                lagpolynomial = eye(K)+lagpolynomial; % lag polynomial, A(1) = I_K - A_1 - A_2 - ... - A_p.
                
                invA1 = inv(lagpolynomial); % Inverse of lag polynomial, A(1)^-1.
            end
            
            % Store the updated B and Lambda matrix values. Needs to be inside the loop!
            B = Bhat;
            Lambda = Lambdahat;
            U = U_hat;
            
            % Convergence criteria and maximum number of iterations.
            conv2 = -(Fval(j+1) - Fval(j));% since the fval is negative

            if exit == 1 || j >= 100
                conv2 = 0;
            end
            
            j=j+1;
        end
        
        %% Store new estimates of Expectation and Maximization step and repeat iteration
        % This is done outside the inner loop.
        
        Sigma = Sigmahat;
        P1 = P2;
        xi1 = xiT(:,1);
        
        % Store estimated values of each iteration.
        Loglikelihood(n+1,1) = loglikelihood;
        
        % Convergence crieteria for main loop.
        conv = abs(Loglikelihood(n+1,1)-Loglikelihood(n,1));
        
        if exit == 1 || n >= 500
            conv = 0;
        end
        
        n=n+1 %#ok<NOPRT>
    end
    
    % Store the maximum log-likelihood and parameter estimates
    % from the current round of iterations on the outer loop.
    if exit == 1
        storeloglike(o,1) = -inf;
    else
        storeloglike(o,1) = loglikelihood;
    end
    
    Parameters(:,o) = pars;    
end

% Get the maximum of the log-likelihoods from all iterations on the big
% loop, and the corresponding smoothed probabilities.
storeloglike(storeloglike==0)=-inf;
[llike_restr, I] = max(storeloglike);
pars = Parameters(:,I);

end

function y = objfun(pars,T,U,XIhat,That,K,M,intercept,slope)
% This is the log-likelihood function for calculating the
% parameters of the covariance matrices.

B = reshape(pars(1:K^2),K,K);
Lambda = zeros(K,K,M);
A = Lambda;
invSigma = Lambda;
Lambda(:,:,1) = eye(K);
for m = 2:M
    Lambda(:,:,m) = diag(pars(K^2+1+K*(m-2):K^2+K*(m-1)));
end

for m = 1:M
    if slope == 'n' && intercept == 'n'
        A(:,:,m) = U*XIhat(:,:,m)*U';
    else
        A(:,:,m) = U(:,:,m)*XIhat(:,:,m)*U(:,:,m)';
    end
    invSigma(:,:,m) = inv(B*Lambda(:,:,m)*B');
end

y = (T)*log(abs(det(B)))+(1/2)*trace(invSigma(:,:,1)*A(:,:,1));

for m = 2:M
    y = y + (1/2)*That(m,:)*log(det(Lambda(:,:,m))) + (1/2)*trace(invSigma(:,:,m)*A(:,:,m));
end

end