% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [LRtest,Pars_lr] = lrtestsvecm(y,r,p,T,K,M,alpha,beta,long_restr,short_restr,Xi_tilde,trend,loglike_ms,intercept,beta_est)
% This function gives the Likelihood Ratio (LR) test reults for joint equality
% of the Lambda matrices parameters. It is generalized to states and
% number of variables.
% The LR tests have the form of lambda_11,1 = lambda_22,1, ... , lambda_11,M = lambda_22,M,
% and then lambda_11,1 = lambda_33,1, ... , lambda_11,M = lambda_33,M, etc.

% Inputs
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables (output of svecm.m)
    % T: number of observations not taken as given (output of svecm.m)
% r: cointegration rank
% p: number of lags as in the VAR form of the VECM
% M: number of Markov states
% alpha - (Kxr) loadings matrix (output of svecm.m)
% beta - (Kxr) matrix of cointegration relationships (output of svecm.m)
% long_restr: matrix with long-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere
% short_restr: same as long_restr for restrictions directly on B
% Xi_tilde - (KxK) matrix from Granger's representation theorem (output of svecm.m)
% trend: if trend = 'y', D_t = nu1 + nu2*t, otherwise D_t = nu1 (see svecm.m). Note, the trend term does not switch.
% loglike_ms: log-likelihood of the Markov switching MS-SVEC model (output of mssvecm.m)
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise. Note, there is always an intercept term in the model.
% beta_est: if beta_est = 'y' the beta matriy is estimated in each optimization round, if beta_est = 'n' the original beta matrix from svecm.m is kept throughout.

% Outputs
% Lrtest - (Kx(K-1)/2x3) matrix of log-likelihood, LR test and p-values.
% Pars_lr - ((K^2 + K*(M-1))x(K-1)/2) matrix of covariance decomposition parameters, B and Lambda_i, i=2,...,M.

%% Define the Restriction matrix
A = [ones(K-1,1) -eye(K-1,K-1)];
BB = A;

i = 2;
while i < K
    Z = [zeros(K-i,i-1) BB(1:K-i,1:K-i+1)];
    A = [A; Z]; %#ok<AGROW>
    i = i + 1;
end

Restrictions = zeros((M-1)*K*(K-1)/2,K*(M-1));
for m = 1:M-1
    Restrictions(K*(K-1)/2*(m-1)+1:K*(K-1)/2*m,K*(m-1)+1:K*m) = A;
end
Restrictions = [zeros((M-1)*K*(K-1)/2,K^2) Restrictions];

% This matrix is used for the fmincon function and is used in particular for
% the nonlinear equality constraint in nonlcon such that Ceq(pars) = 0.
% In our case it is Ceq*pars = 0, where pars are the Lambda matrix
% parameters.
Ceq = zeros(M-1,size(Restrictions,2),K*(K-1)/2);
j = 1;
for m =1:M-1
    for i = 1:size(Ceq,3)
        Ceq(m,:,i) = Restrictions(j,:);
        j = j + 1;
    end
end

% This matrix ensures that the same optimum is not reached at different
% tests. It is an inequality contraint such that the values being compared
% are different from other values.
if K <= 2
    C = [];
else
    C = zeros(size(Ceq));
    h = 1;
    k = K - 1;
    j = 1;
    while h <= size(Ceq,3)
        i = 1;
        while i <= k
            if h ~= j
                C(:,:,h) = Ceq(:,:,j);
            elseif h == j
                if j == size(Ceq,3)
                    C(:,:,h) = Ceq(:,:,j-1);
                else
                    C(:,:,h) = Ceq(:,:,j+1);
                end
            end
            h = h + 1;
            i = i + 1;
        end
        j = j + k;
        k = k - 1;
    end
end

%% Obtain the LR test results
% These take the form of l_11,1 = l_22,1, l_11,2 = l_22,2, etc

df = M - 1;
Pars_lr = zeros(K^2 + K*(M-1),1);
LRtest = zeros(size(Ceq,3),2);
Llike = zeros(size(Ceq,3),1);
for i = 1:size(Ceq,3)
    ceq = @(x) Ceq(:,:,i)*x;
    c = @(x) C(:,:,i)*x;
    [llike_restr,pars] = llike(y,r,p,T,K,M,alpha,beta,long_restr,short_restr,Xi_tilde,trend,ceq,c,intercept,beta_est);
    Llike(i) = llike_restr;
    LRtest(i,1) = 2*(loglike_ms - Llike(i));
    LRtest(i,2) = 1 - chi2cdf(LRtest(i,1),df);
    Pars_lr(:,i) = pars;
end
LRtest = [Llike LRtest];

display('The restricted log-likelihood value, the Likelihood Ratio test and p value are')
display(LRtest)

end

function [llike_restr,pars] = llike(y,r,p,T,K,M,alpha,beta,long_restr,short_restr,Xi_tilde,trend,ceq,c,intercept,beta_est)
%% Set up exogenous matrices
Deltay = diff(y,1,2);
DeltaY = Deltay(:,p:end);
Yminus1 = y(:,p:end-1);
bold_y = reshape(DeltaY,T*K,1);
[Xbar] = xbar(r,p,T,K,M,beta,Yminus1,Deltay,intercept,trend,beta_est);

%% Initialize storing matrices

storeloglike = zeros(1,1);
Parameters = zeros(K^2 + K*(M-1),1);

for o = 1:100
    o %#ok<NOPRT>
    
    if beta_est == 'n'
        beta_orth = null(beta','r');
        if intercept == 'n'
            theta = (kron(Xbar'*Xbar,eye(K)))\kron(Xbar',eye(K))*bold_y; % Replaced inv(kron(Xbar'*Xbar,eye(K)))*
            U = bold_y-kron(Xbar,eye(K))*theta;
            U = reshape(U,K,T); % (KxT) matrix of residuals
            B = sqrtm(U*U'/T) + randn(K,K)/100;
        else % model with switching intercept
            A = 0; AA = 0;
            for m = 1:M
                A = A + kron(Xbar(:,:,m)'*Xbar(:,:,m),eye(K));
                AA = AA + kron(Xbar(:,:,m)',eye(K));
            end
            theta = A\AA*bold_y; % replaced inv(A)*
            
            XXbar = Xbar(:,M:end,M);
            U = zeros(1,K*T,M);
            for m = 1:M
                U(:,:,m) = bold_y - kron(XXbar,eye(K))*[theta(1+K*(m-1):K*m,1); theta(K*M+1:end,1)];
            end
            U = reshape(U,K,T,M);
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/100;
        end
    else % beta estimated by RR regression
        Pi = -alpha*beta';
        if intercept == 'n'
            theta = (kron(Xbar'*Xbar,eye(K)))\kron(Xbar',eye(K))*bold_y;
            theta = reshape(theta,K,(size(theta,1))/K);
            U = DeltaY - Pi*Yminus1 - theta*Xbar';
            B = sqrtm(U*U'/T) + randn(K,K)/100;
        else % model with switching intercept
            XXbar = Xbar(:,M:end,M);
            theta = (kron(XXbar'*XXbar,eye(K)))\kron(XXbar',eye(K))*bold_y;
            theta = reshape(theta,K,(size(theta,1))/K);
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = DeltaY - Pi*Yminus1 - theta*XXbar';
            end
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/100;
        end
    end
    
    % Starting values for the Lambda matrix
    Lambda = zeros(K,K,M);
    Lambda(:,:,1) = eye(K);
    for m = 2:M
        Lambda(:,:,m) = o^(m-1)*eye(K);
    end
    
    % Covariance matrices
    Sigma = zeros(K,K,M);
    for m = 1:M
        Sigma(:,:,m) = B*Lambda(:,:,m)*B';
    end
    
    % Starting value of the transition probabilities. Note probabilities add up to 1 row-wise.
    P1 = ones(M,1)*ones(1,M)/M;
    % Starting value of the initial xit_0|0 vector
    xi1 = ones(M,1)/M;
    
    % Store the log-likelihood values:
    Loglikelihood = zeros(1,1);

    %% Main Program using the EM algorithm
    n = 1 %#ok<NOPRT>
    conv1 = 1;
    while conv1 > 10^-6
        %% Expectation Step
        
        [loglikelihood,xiT,P2] = expectation_mssvecm(T,K,M,Sigma,U,xi1,P1,intercept);
        %% Maximization Step
        
        XIhat = zeros(T,T,M);
        for m = 1:M
            XIhat(:,:,m) = diag(xiT(m,2:end));
        end
        
        That = sum(xiT(:,2:T+1),2);
        
        % Lower bound for the Lambda parameters for fmincon
        lb = -inf(K^2 + K*(M-1),1);
        lb(end-K*(M-1)+1:end,1) = 0.001;
        
        % Options displaying results.
        options = optimset('display','final','Algorithm', 'interior-point','MaxFunEvals', 20000, ...
            'MaxIter', 20000, 'DerivativeCheck','on','Diagnostics','off','GradObj','off','LargeScale','off');
        
        j = 1;
        Fval = zeros(1,1);
        Pars = zeros(K^2 + K*(M-1),1);
        conv2 = 1;
        while conv2 > 10^-6
            
            Lambdapars = zeros(K*(M-1),1);
            for m = 2:M
                Lambdapars(1+K*(m-2):K*(m-1),:) = diag(Lambda(:,:,m));
            end
            
            pars0 = [B(:); Lambdapars];
            
            if K <= 2
                nonlcon = @(pars0) deal([], ceq(pars0));
            else
                nonlcon = @(pars0) deal(c(pars0) ,ceq(pars0));
            end
            
            % Estimate the B and Lambda parameters.
            if size(short_restr,1) + size(long_restr,1) == 0 % unrestricted case:
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept), pars0,[],[],[],[],lb,[],nonlcon,options);
            else
                if size(long_restr,1) == 0 % restricted case according to restriction:
                    Aeq = short_restr;
                elseif size(short_restr,1) == 0
                    long_restr_updated = long_restr*kron(eye(K),Xi_tilde);
                    Aeq = long_restr_updated;
                else
                    long_restr_updated = long_restr*kron(eye(K),Xi_tilde);
                    Aeq = [long_restr_updated; short_restr];
                end
                C = [Aeq zeros(size(Aeq,1),K*(M-1))];
                beq = zeros(size(C,1),1);
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept), pars0,[],[],C,beq,lb,[],nonlcon,options);
            end
            
            Fval(j+1,1) = fval;
            Pars(:,j) = pars;
            
            % Return the parameter estimates.
            Bhat = reshape(pars(1:K^2),K,K);
            Lambdahat = zeros(K,K,M);
            Lambdahat(:,:,1) = eye(K);
            for m = 2:M
                Lambdahat(:,:,m) = diag(pars(K^2+1+K*(m-2):K^2+K*(m-1)));
            end
            
            Sigmahat = zeros(K,K,M);
            for m = 1:M
                Sigmahat(:,:,m) = Bhat*Lambdahat(:,:,m)*Bhat';
            end
            
            exit = singlularity_check(Sigmahat,K,M);
            
            % Estimate of the remaining parameters. This is done according to
            % model specification
            A = 0; AA = 0;
            if beta_est == 'n'
                if intercept == 'n'
                    for m = 1:M
                        A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    
                    thetahat = A\AA*bold_y;
                    U_hat = bold_y-kron(Xbar,eye(K))*thetahat;
                    U_hat = reshape(U_hat,K,T);
                    
                    if size(long_restr,1) > 0 % formulate the long-run impact matrix, Xi_tilde
                        if trend == 'n'
                            alpha = reshape(thetahat(K + 1:K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(1+r) + 1:K*(1+r) + K^2*(p-1),1),K,K*(p-1));
                        else
                            alpha = reshape(thetahat(2*K + 1:2*K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(2+r) + 1:K*(2+r) + K^2*(p-1),1),K,K*(p-1));
                        end
                        
                        alpha_orth = null(alpha','r');
                        lagpolynomial = zeros(K,K);
                        for i = 1:p-1
                            lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
                        end
                        lagpolynomial = eye(K) + lagpolynomial;
                        Xi_tilde_hat = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';
                        Xi_tilde = Xi_tilde_hat;
                    end
                else % model with switching intercept
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    
                    thetahat = A\AA*bold_y;
                    U_hat = zeros(1,K*T,M);
                    for m = 1:M
                        U_hat(:,:,m) = bold_y - kron(XXbar,eye(K))*[thetahat(1+K*(m-1):K*m,1); thetahat(K*M+1:end,1)];
                    end
                    U_hat = reshape(U_hat,K,T,M);
                    
                    if size(long_restr,1) > 0 % formulate the long-run impact matrix, Xi_tilde
                        if trend == 0
                            alpha = reshape(thetahat(M*K + 1:M*K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(M+r) + 1:K*(M+r) + K^2*(p-1),1),K,K*(p-1));
                        else
                            alpha = reshape(thetahat((1+M)*K + 1:(1+M)*K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(1+M+r) + 1:K*(1+M+r) + K^2*(p-1),1),K,K*(p-1));
                        end
                        
                        alpha_orth = null(alpha','r');
                        lagpolynomial = zeros(K,K);
                        for i = 1:p-1
                            lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
                        end
                        lagpolynomial = eye(K) + lagpolynomial;
                        Xi_tilde_hat = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';
                        Xi_tilde = Xi_tilde_hat;
                    end
                end
            else % beta estimated by RR regression
                if intercept == 'n'
                    for m = 1:M
                        A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    Pihat = A\AA*bold_y; Nhat = A\AA*reshape(Yminus1,T*K,1);
                    Pihat = reshape(Pihat,K,size(Pihat,1)/K); Nhat = reshape(Nhat,K,size(Nhat,1)/K);
                    Uhat = DeltaY - Pihat*Xbar'; Vhat = Yminus1 - Nhat*Xbar';
                else % model with switching intercept
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    Pihat = A\AA*bold_y; Nhat = A\AA*reshape(Yminus1,T*K,1);
                    Pihat = reshape(Pihat,K,size(Pihat,1)/K); Nhat = reshape(Nhat,K,size(Nhat,1)/K);
                    Uhat = zeros(K,T*M); Vhat = Uhat;
                    for m = 1:M
                        Uhat(:,1+(m-1)*T:m*T) = DeltaY - Pihat*Xbar(:,:,m)'; Vhat(:,1+(m-1)*T:m*T) = Yminus1 - Nhat*Xbar(:,:,m)';
                    end
                    
                end
                SIGMAvv = Vhat*Vhat'/T; SIGMAuu = Uhat*Uhat'/T; SIGMAuv = Uhat*Vhat'/T; SIGMAvu = SIGMAuv';
                % Check for stopping the algorithm:
                not_num = zeros(K,K,3); infin = not_num;
                not_num(:,:,1) = isnan(SIGMAvv); not_num(:,:,2) = isnan(SIGMAuu); not_num(:,:,3) = isnan(SIGMAuv);
                not_num = sum(not_num(:));
                infin(:,:,1) = isinf(SIGMAvv); infin(:,:,2) = isinf(SIGMAuu); infin(:,:,3) = isinf(SIGMAuv);
                infin = sum(infin(:));
                if not_num + infin > 0
                    exit = 1;
                else
                    Low_eigs = zeros(K,1,3);
                    Low_eigs(:,:,1) = eig(SIGMAvv); Low_eigs(:,:,2) = eig(SIGMAuu); Low_eigs(:,:,3) = abs(eig(SIGMAuv));
                    Low_eigs = Low_eigs < 0.00001;
                    Low_eigs = sum(Low_eigs(:));
                    if Low_eigs > 0
                        exit = 1;
                        % Stopping check done
                    else
                        [a lambda] = eigs(SIGMAvv\SIGMAvu/(SIGMAuu)*SIGMAuv,r); %#ok<NASGU>
                        beta_b = zeros(size(a));
                        for i = 1:size(a,2)
                            beta_b(:,i) = a(:,i)/(a(:,i)'*SIGMAvv*a(:,i))^0.5; % [20.2.11]
                        end
                        beta_r = beta_b(1:r,:); % Normalization as in Ltkepohl (2005)
                        beta = beta_b/beta_r; % the beta matrix
                        
                        alpha = -SIGMAuv*beta_b*beta_r'; % the alpha matrix
                        
                        Pi = -alpha*beta'; % estimated Pi matrix
                        
                        % Parameter matrices:
                        if intercept == 'n'
                            if trend == 'n'
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+2:K*i+1) - Pi*Nhat(:,K*(i-1)+2:K*i+1);
                                end
                                nu1 = Pihat(:,1) - Pi*Nhat(:,1); % constant term
                                
                                Gam = [nu1 Gamma];
                                U_hat = DeltaY - Pi*Yminus1 - Gam*Xbar';
                            else
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+3:K*i+2) - Pi*Nhat(:,K*(i-1)+3:K*i+2);
                                end
                                nu1 = Pihat(:,1) - Pi*Nhat(:,1); % constant term
                                nu2 = Pihat(:,2) - Pi*Nhat(:,2); % trend term
                                
                                Gam = [nu1 nu2 Gamma];
                                U_hat = DeltaY - Pi*Yminus1 - Gam*Xbar';
                            end
                        else % model with switching intercept
                            if trend == 'n'
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+M+1:K*i+M) - Pi*Nhat(:,K*(i-1)+M+1:K*i+M);
                                end
                                nu1 = Pihat(:,1:M) - Pi*Nhat(:,1:M); % constant terms
                                
                                Gam = [nu1 Gamma];
                                U_hat = zeros(K,T,M);
                                for m = 1:M
                                    U_hat(:,:,m) = DeltaY - Pi*Yminus1 - Gam*Xbar(:,:,m)';
                                end
                            else
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+M+2:K*i+M+1) - Pi*Nhat(:,K*(i-1)+M+2:K*i+M+1);
                                end
                                nu1 = Pihat(:,1:M) - Pi*Nhat(:,1:M); % constant terms
                                nu2 = Pihat(:,M+1) - Pi*Nhat(:,M+1); % trend term
                                
                                Gam = [nu1 nu2 Gamma];
                                U_hat = zeros(K,T,M);
                                for m = 1:M
                                    U_hat(:,:,m) = DeltaY - Pi*Yminus1 - Gam*Xbar(:,:,m)';
                                end
                            end
                        end
                        beta_orth = null(beta','r');
                    end
                end
            end
            
            % Store the updated matrices
            B = Bhat;
            Lambda = Lambdahat;
            U = U_hat;
            
            % Convergence criteria and maximum number of iterations.
            conv2 = -(Fval(j+1,1)-Fval(j,1));
            
            if exit == 1 || j >= 100
                conv2 = 0;
            end
            
            j = j + 1;
        end
        
        %% Store new estimates of Expectation and Maximization step and repeat iteration
        
        Sigma = Sigmahat;
        P1 = P2;
        xi1 = xiT(:,1);
        
        Loglikelihood(n+1,1) = loglikelihood;
        
        conv1 = abs(Loglikelihood(n+1,1)-Loglikelihood(n,1));
        
        if exit == 1 || n >= 1000
            conv1 = 0;
        end
        
        n = n + 1 %#ok<NOPRT>
    end
    
    % Store the maximum log-likelihood and parameter estimates
    % from the current round of iterations on the outer loop.
    if exit == 1
        storeloglike(o,1) = -inf;
    else
        storeloglike(o,1) = loglikelihood;
    end
    
    Parameters(:,o) = pars;
end

% Get the maximum of the log-likelihoods from all iterations on the big
% loop, and the corresponding smoothed probabilities.
storeloglike(storeloglike==0)=-inf;
[llike_restr, I] = max(storeloglike);
pars = Parameters(:,I);

end

function y = objfun(pars,T,U,XIhat,That,K,M,intercept)
% This is the log-likelihood function for calculating the (unrestricted)
% parameters of the B and Lambda matrices.

B = reshape(pars(1:K^2),K,K);
Lambda = zeros(K,K,M);
A = Lambda;
invSigma = Lambda;
Lambda(:,:,1) = eye(K);
for m = 2:M
    Lambda(:,:,m) = diag(pars(1 + K^2 + K*(m-2):K^2 + K*(m-1)));
end

for m = 1:M
    if intercept == 'n'
        A(:,:,m) = U*XIhat(:,:,m)*U';
    else
        A(:,:,m) = U(:,:,m)*XIhat(:,:,m)*U(:,:,m)';
    end
    invSigma(:,:,m) = inv(B*Lambda(:,:,m)*B');
end

y = T*log(abs(det(B)))+(1/2)*trace(invSigma(:,:,1)*A(:,:,1));

for m = 2:M
    y = y + (1/2)*That(m,:)*log(det(Lambda(:,:,m))) + (1/2)*trace(invSigma(:,:,m)*A(:,:,m));
end

end

function exit = singlularity_check(Sigmahat,K,M)

% Check the eigenvalues of the covariance matrices. If smaller
% than a certain number exit and start from the beginning.
exit = 0;
not_number = isnan(Sigmahat);
infinite = isinf(Sigmahat);
not_number = sum(not_number(:));
infinite = sum(infinite(:));
if not_number + infinite > 0
    exit = 1;
else
    Eigenvalues_Sigmahat = zeros(K,1,M);
    for m = 1:M
        Eigenvalues_Sigmahat(:,1,m) = eig(Sigmahat(:,:,m));
    end
    low_eigenvalues = Eigenvalues_Sigmahat < 0.00000001;
    low_eigenvalues = sum(low_eigenvalues(:));
    if low_eigenvalues > 0
        exit = 1;
    end
end

end