% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [K,T,loglike_ms,gamma,cov] = mssvar(p,M,y,range,short_restr,long_restr,intercept,slope)
% This function estimates the (restricted) MS-SVAR model parameters and their
% standard errors. The model being estimated is:
% y_t = nu1(s_t) + A_1(s_t)*y_t-1 + A_2(s_t)*y_t-2 + ... + A_p(s_t)*y_t-p + u_t,
% such that y_t is a stationary process and u_t ~
% N(0,Sigma(s_t)), according to a
% stochastic state s_t goverened by a first-order Markov process. The
% covaraince matrix always switches. All other parameters do so optionally.
% The errors can be decomposed as u_t = B*eps_t, where B is the
% short-run, (contemporaneous) matrix and restrictions can be imposed on
% it, given by short_restr, and/or long-run restrictions on matrix
% A(1)^-1*B, given by long_restr.

% MODEL DESCRIPTION:
% - MSH(M)-SVAR(p): switching covariance matrix
% - MSIH(M)-SVAR(p): switching covariance matrix and intercept term
% - MSAH(M)-SVAR(p): switching covariance matrix and AR coefficients
% - MSIAH(M)-SVAR(p): switching covariance matrix, intercept term and AR coefficients
% For the intercept term to switch, intercept = 'y', for the AR coeffiecints
% to switch, slope = 'y'. IMPORTANT: use 'n' otherwise!

% INPUTS:
% p - number of lags
% M - number of Markov states
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables (output)
    % T: number of observations not taken as given (output)
% range - (1x(T+p)) vector of dates used for horizontal axis in smoothed probabilities plot. If range = 0 no dates plotted.
% short_restr: matrix with short-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere
% long_restr - same as short_restr for restrictions on A(1)^-1*B
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise.
% slope: slope = 'y' for a switching AR matrices, slope = 'n' otherwise.

% OUTPUTS:
% K - number of endogenous variables
% T - number of observations  not taken as given
% loglike_ms - log-likelihood of the Markov switching (MS) SVAR model
% gamma - matrix with column of parameter estimates and colum of their respective standard errors (see code for details).
% cov - covariance matrix of model parameters. Dimension depends on the type of model specified.

%% Set up exogenous matrices

[Y,K,T,Zbar,Xbar,bold_y,z] = initialization(y,p,M,intercept,slope);

%% Initialize storing matrices

storeloglike = zeros(1,1);
KsiT = zeros(M,T,1);
Gamma = zeros(z + K^2 - size(short_restr,1) + M*(M-1) + K*(M-1),1,1);
COV = zeros(K,K,M,1);
PP = zeros(M,M,1);

for o = 1:100
    o %#ok<NOPRT>
    
    %% Initial Values
    % beta is a vector of intercept and vectorized slope parameters
    if slope == 'n'
        if intercept == 'n' % MSH(M)-SVAR(p) Model
            beta = kron(Xbar'*Xbar,eye(K))\kron(Xbar',eye(K))*bold_y; % replaced inv(A)*b by A\b
            U = bold_y - kron(Xbar,eye(K))*beta;
            U = reshape(U,K,T);
            B = sqrtm(U*U'/T) + randn(K,K)/100; % Starting value for the B matrix
            Apars = reshape(beta(K+1:end),K,K*p);
        else % MSIH(M)-SVAR(p) Model
            XX = Xbar(:,M:end,M);

            beta = (XX'*XX)\XX'*Y';
            
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = Y - beta'*XX';
            end

            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
            Apars = beta(2:end,:)';
        end
    else
        if intercept == 'n' % MSAH(M)-SVAR(p) Model
            XX = Xbar(:,1:1+K*p,1);
            
            beta = (XX'*XX)\XX'*Y';
            
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = Y - beta'*XX';
            end
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
            Apars = beta(2:end,:)';
        else % MSIAH(M)-SVAR(p) Model
            Beta = zeros(K*(K*p+1),M);
            U = zeros(K*T,M);
            for m = 1:M
                Beta(:,m) = kron(Xbar'*Xbar,eye(K))\kron(Xbar',eye(K))*bold_y;
                U(:,m) = bold_y-kron(Xbar,eye(K))*Beta(:,m);
            end
            U = reshape(U,K,T,M);
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
            Apars = reshape(Beta(K+1:end,1),K,K*p);
        end
    end
    lagpolynomial = zeros(K,K);
    for i = 1:K:K*p
        lagpolynomial = lagpolynomial - Apars(:,i:i+K-1);
    end
    lagpolynomial = eye(K)+lagpolynomial; % lag polynomial, A(1) = I_K - A_1 - A_2 - ... - A_p.
    invA1 = inv(lagpolynomial); % Inverse of lag polynomial, A(1)^-1.
    
    Lambda = zeros(K,K,M);
    Lambda(:,:,1) = eye(K); % Starting value for the Lambda matrix
    for m = 2:M
        Lambda(:,:,m) = o^(m-1)*eye(K);
    end
    
    Sigma = zeros(K,K,M);
    for m = 1:M
        Sigma(:,:,m) = B*Lambda(:,:,m)*B';
    end
    
    P1 = ones(M,1)*ones(1,M)/M; % Starting value of the transition probabilities. Note
    % probabilities add up to 1 row-wise.
    xi1 = ones(M,1)/M; % Starting value of the initial xi vector
    
    % Store the log-likelihood values per iteration for convergence criteria.
    Loglikelihood = zeros(1,1);
    
    % Exit condition, to be used in case of singulariites
    exit = 0;
    %% Main Program using the EM algorithm
    n = 1 %#ok<NOPRT>
    conv = 1;
    while conv>10^-6
        %% Expectation Step
        
        [loglikelihood,xiT,P2] = expectation_mssvar(T,K,M,Sigma,U,xi1,P1,intercept,slope);
        
        %% Maximization Step
        
        % Define the matrices, these are from the expectation step and stay
        % outside of the maximization loop.
        
        XIhat = zeros(T,T,M);
        for m = 1:M
            XIhat(:,:,m) = diag(xiT(m,2:end));
        end
        
        That = sum(xiT(:,2:T+1),2);
        
        %% Estimate the Covariance Matrices and the remaining Parameters
        % This is a loop where first the covariance matrix parameters are
        % estimated through the log-likelihood function objfun and then these
        % values are plugged into the equation to estimate the intercept and
        % slope parameters, betahat as in Krolzig. From betahat, the new
        % residual matrix, U is estimated and this is plugged back into the
        % function objfun. This loop continues until a covergence criteria is
        % met. Restrictions on the B matrix can now be set through
        % long-restr.
        
        % Options displaying results.
        options = optimset('display','final','Algorithm', 'interior-point','MaxFunEvals', 20000, ...
            'MaxIter', 20000, 'Hessian','bfgs','DerivativeCheck','on','Diagnostics','off','GradObj','off','LargeScale','off');
        
        % Lower bound for the Lambda parameters for fmincon
        lb = -inf(K^2 + K*(M-1),1);
        lb(end-K*(M-1)+1:end,1) = 0.001;
        
        j = 1;
        Fval = zeros(1,1); % Matrix storing the values of the log-likelihood function.
        Pars = zeros(K^2 + K*(M-1),1); % Stores the values of the parameter estimates.
        conv2 = 1;
        
        while conv2>10^-6
            
            % Starting value of the parameters to be estimated using a non-linear
            % estimator, fmincon. Parameters are stored in a vector pars0 and
            % are updated at each iteration.
            
            Lambdapars = zeros(K*(M-1),1);
            for m = 2:M
                Lambdapars(1+K*(m-2):K*(m-1),:) = diag(Lambda(:,:,m));
            end
            
            pars0 = [B(:); Lambdapars];
            
            % Set up restrictions and estimate the B and Lambda parameters.
            if size(long_restr,1) + size(short_restr,1) == 0 % unrestricted case
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept,slope), pars0,[],[],[],[],lb,[],[],options);
            else % Models with restrictions
                if size(long_restr,1) == 0 % only short-run restrictions
                    aeq = short_restr;
                elseif size(short_restr,1) == 0 % only long-run restrictions
                    aeq = long_restr*kron(eye(K),invA1);
                else % long and short-run restrictions
                    long_restr_updated = long_restr*kron(eye(K),invA1);
                    aeq = [long_restr_updated; short_restr];
                end
                Aeq = [aeq zeros(size(aeq,1),K*(M-1))];
                beq = zeros(size(Aeq,1),1);
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept,slope), pars0,[],[],Aeq,beq,lb,[],[],options);
            end
            
            Fval(j+1) = fval;
            Pars(:,j) = pars;
            
            % Return the parameter estimates.
            Bhat = reshape(pars(1:K^2),K,K);
            Lambdahat = zeros(K,K,M);
            Lambdahat(:,:,1) = eye(K);
            for m = 2:M
                Lambdahat(:,:,m) = diag(pars(K^2+1+K*(m-2):K^2+K*(m-1)));
            end
            
            Sigmahat = zeros(K,K,M);
            for m = 1:M
                Sigmahat(:,:,m) = Bhat*Lambdahat(:,:,m)*Bhat';
            end
            
            % Check the eigenvalues of the covariance matrices. If smaller
            % than a certain number exit and start from the beginning.
            not_number = isnan(Sigmahat);
            infinite = isinf(Sigmahat);
            not_number = sum(not_number(:));
            infinite = sum(infinite(:));
            if not_number + infinite > 0
                exit = 1;
            else
                Eigenvalues_Sigmahat = zeros(K,1,M);
                for m = 1:M
                    Eigenvalues_Sigmahat(:,1,m) = eig(Sigmahat(:,:,m));
                end
                low_eigenvalues = Eigenvalues_Sigmahat < 0.0000001;
                low_eigenvalues = sum(low_eigenvalues(:));
                if low_eigenvalues > 0
                    exit = 1;
                end
            end
            
            % Estimate the intercept and slope parameters according to
            % Table 9.12 p.192 of Krolzig(1997)
            A = zeros(z,z);
            AA = zeros(z,K*T);
            if slope == 'n'
                if intercept == 'n' % MSH(M)-SVAR(p) Model
                    for m = 1:M
                        A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    betahat = A\AA*bold_y;
                    U_hat = bold_y-kron(Xbar,eye(K))*betahat;% The residual matrix.
                    U_hat = reshape(U_hat,K,T);
                    Apars = reshape(betahat(K+1:end),K,K*p); % store the estimated slope parameters
                else % MSIH(M)-SVAR(p) Model
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    betahat = A\AA*bold_y;
                    U_hat = zeros(1,K*T,M); % Residual matrices with switching intercept term
                    for m = 1:M
                        U_hat(:,:,m) = bold_y - kron(XX,eye(K))*[betahat(1+K*(m-1):K*m,1); betahat(K*M+1:end,1)];
                    end
                    U_hat = reshape(U_hat,K,T,M);
                    Apars = reshape(betahat(M*K+1:end),K,K*p);
                end
            else % MSAH(M)-SVAR(p) Model
                if intercept == 'n'
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    betahat = A\AA*bold_y;
                    U_hat = zeros(K,T,M);
                    for m = 1:M
                        U_hat(:,:,m) = Y - reshape(betahat(1+K+K^2*p*(m-1):K+K^2*p*m),K,K*p)*Zbar' - kron(ones(T,1)',betahat(1:K));
                    end
                    A1 = 0; AA1 = A1;
                    for m = 1:M
                        A1 = A1 + kron(XX'*XIhat(:,:,m)*XX,inv(Sigmahat(:,:,m)));
                        AA1 = AA1 + kron(XX'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    Apars = A1\AA1*bold_y;
                    Apars = reshape(Apars(K+1:end),K,K*p);
                else % MSIAH(M)-SVAR(p) Model
                    Betahat = zeros(z/M,M);
                    for m = 1:M
                        Betahat(:,m) = kron((Xbar'*XIhat(:,:,m)*Xbar)\Xbar'*XIhat(:,:,m),eye(K))*bold_y;
                    end
                    U_hat = zeros(1,K*T,M);
                    for m = 1:M
                        U_hat(:,:,m) = bold_y-kron(Xbar,eye(K))*Betahat(:,m);
                    end
                    U_hat = reshape(U_hat,K,T,M);
                    A1 = 0; AA1 = A1;
                    for m = 1:M
                        A1 = A1 + Xbar'*XIhat(:,:,m)*Xbar;
                        AA1 = AA1 + Xbar'*XIhat(:,:,m);
                    end
                    Apars = kron(A1\AA1,eye(K))*bold_y;
                    Apars = reshape(Apars(K+1:end),K,K*p);
                end
            end

            if size(long_restr,1) > 0
                % Get the new value of the lagpolynomial.
                lagpolynomial = zeros(K,K);
                for i = 1:K:K*p
                    lagpolynomial = lagpolynomial - Apars(:,i:i+K-1);
                end
                lagpolynomial = eye(K)+lagpolynomial; % lag polynomial, A(1) = I_K - A_1 - A_2 - ... - A_p.
                
                invA1 = inv(lagpolynomial); % Inverse of lag polynomial, A(1)^-1.
            end
            
            % Store the updated B, Lambda and U matrix values.
            B = Bhat;
            Lambda = Lambdahat;
            U = U_hat;
            
            % Convergence criteria and maximum number of iterations.
            conv2 = -(Fval(j+1) - Fval(j));% since the fval is negative
            
            if exit == 1 || j >= 100
                conv2 = 0;
            end
            
            j=j+1;
        end
        
        %% Store new estimates of Expectation and Maximization step and repeat iteration
        % This is done outside the inner loop.
        
        Sigma = Sigmahat;
        P1 = P2;
        xi1 = xiT(:,1);
        
        % Store estimated values of each iteration.
        Loglikelihood(n+1,1) = loglikelihood;
        
        % Convergence crieteria for main loop.
        conv = abs(Loglikelihood(n+1,1)-Loglikelihood(n,1));
        
        if exit == 1 || n >= 1000
            conv = 0;
        end
        
        n = n + 1 %#ok<NOPRT>
    end
    
    % Store the maximum log-likelihood and parameter estimates
    % from the current round of iterations on the outer loop.
    if exit == 1
        storeloglike(o,1) = -inf;
    else
        storeloglike(o,1) = loglikelihood;
    end
    KsiT(:,:,o) = xiT(:,2:T+1);
    COV(:,:,:,o) = Sigma;
    PP(:,:,o) = P1;
    
    vecB = B(:);
    if size(short_restr,1) > 0
        sum_restr = sum(short_restr,1)';
        Bpars = vecB(sum_restr == 0);
    else
        Bpars = vecB;
    end
    
    Lambdapars = zeros(K*(M-1),1);
    for m = 2:M
        Lambdapars(1+K*(m-2):K*(m-1),:) = diag(Lambda(:,:,m));
    end
    
    Ppars = reshape(P1(1:M-1,1:M-1),(M-1)^2,1);
    Ppars(1+(M-1)^2:(M-1)^2+M-1,:) = reshape(P1(M,2:M),M-1,1);
    
    if slope == 'y' && intercept == 'y'
        Gamma(:,1,o) = [Betahat(:); Bpars; Ppars(:); Lambdapars];
    else
        Gamma(:,1,o) = [betahat; Bpars; Ppars(:); Lambdapars];
    end
    
end

% Get the maximum log-likelihood from all iterations on the outer
% loop and conseqently the optimal parameter values from that iteration.
storeloglike(find(storeloglike==0))=-inf;
[loglike_ms, I] = max(storeloglike);
xiT = KsiT(:,:,I);
Sigma = COV(:,:,:,I);
P = PP(:,:,I);
gamma = Gamma(:,1,I);

%% Standard errors
% Once the EM algorithm has converged all parameter estimates are used in the
% log-likelihood function for the whole model, Lik. Standard
% errors are obtained from the (negative of the) inverse of the Hessian matrix at the
% optimum.

% For this function grad is the gradient and hess the hessian
% [grad,hess] = gradhess(gamma,@Lik,'central', K,bold_y,Zbar,Xbar,xi1,T,M,p,z,intercept,slope,short_restr);

% Here hess is the hessian (and err is the error of approximation)
[hess] = hessian(@(gamma) Lik(gamma,K,bold_y,Zbar,Xbar,xi1,T,M,p,z,intercept,slope,short_restr),gamma);

cov = inv(hess); % Covariance matrix
sterr = zeros(size(gamma)); % Standard error matrix
for i = 1:size(gamma,1)
    sterr(i,1) = sqrt(cov(i,i));
end

gamma = [gamma sterr];

%% Plots and Display
% Define recessions according to NBER:
Recessions = [ datenum('15-May-1937'), datenum('15-Jun-1938');
 datenum('15-Feb-1945'), datenum('15-Oct-1945');
 datenum('15-Nov-1948'), datenum('15-Oct-1949');
 datenum('15-Jul-1953'), datenum('15-May-1954');
 datenum('15-Aug-1957'), datenum('15-Apr-1958');
 datenum('15-Apr-1960'), datenum('15-Feb-1961');
 datenum('15-Dec-1969'), datenum('15-Nov-1970');
 datenum('15-Nov-1973'), datenum('15-Mar-1975');
 datenum('15-Jan-1980'), datenum('15-Jul-1980');
 datenum('15-Jul-1981'), datenum('15-Nov-1982');
 datenum('15-Jul-1990'), datenum('15-Mar-1991');
 datenum('15-Mar-2001'), datenum('15-Nov-2001');
 datenum('15-Dec-2007'), datenum('15-Jun-2009') ];
Recessions = busdate(Recessions); % Until the next business day

% Plot probability of the states (and the log-likelihood)

if range == 0
    figure('Name','Smooth probabilities','NumberTitle','off')
    for m = 1:M
        subplot(M,1,m)
        plot(xiT(m,1:T),'LineWidth',2); axis tight;
        ylim([0 1])
    end
else
    range = range(:,p+1:end)';
    range = datenum([range,ones(T,2)]);
    figure('Name','Smooth probabilities','NumberTitle','off')
    for m = 1:M
        subplot(M,1,m)
        plot(range,xiT(m,1:T),'LineWidth',2);
        RecessionPlot(range, Recessions)
        set(gca,'XTick',range(1:20:end)')
        datetick('x','yyyy','keepticks')
        axis tight;
        ylim([0 1])
    end
end

% Return the estimated parameters
if size(short_restr,1) == 0
    B = reshape(gamma(1 + z : z + K^2,1),K,K);
    Bse = reshape(gamma(1 + z : z + K^2,2),K,K);
else
    find_pars = find(sum_restr == 0);
    pars(find_pars) = 1; %#ok<*FNDSB>
    pars = pars';
    Bse = zeros(K,K); B = Bse;
    jj = 1;
    for ii = 1:K^2
        if pars(ii) == 1
            B(ii) = gamma(jj + z,1);
            Bse(ii) = gamma(jj + z,2);
            jj = jj + 1;
        end
    end
end

sr = size(short_restr,1);
Pse = reshape(gamma(1 + z + K^2 - sr:(M-1)^2 + z + K^2 - sr,2),M-1,M-1);
Pse(M,2:M) = gamma(1+(M-1)^2 + z + K^2 - sr: M*(M-1) + z + K^2 - sr,2)';

Lambda = reshape(gamma(1 + M*(M-1) + z + K^2 - sr:M*(M-1) + z + K^2 + K*(M-1) - sr,1),K,M-1);
Lambdase = reshape(gamma(1 + M*(M-1) + z + K^2 - sr:M*(M-1) + z + K^2 + K*(M-1) - sr,2),K,M-1);

% Display results
display('The number of free parameters are')
n=size(gamma,1);
display(size(gamma,1))
display('The maximum log-likelihood is')
fprintf('%10.6f',loglike_ms)
fprintf('\n')
display('The AIC value is')
fprintf('%10.6f ',-2*(loglike_ms-n));
fprintf('\n')
display('The SIC value is')
fprintf('%10.6f',-2*loglike_ms+log(T)*n);
fprintf('\n')
display('The B matrix is')
display(B)
display('The standard errors for the B matrix are')
display(Bse)
if size(long_restr,1) > 0
    display('The long-run impact matrix is')
    display(invA1*B)
end
display('The covariance matrices are')
display(Sigma*1000)
display('The tansition probabilities are')
display(P)
display('The tansition probabilities standard errors are')
display(Pse)
display('The Lambda parameters are')
display(Lambda)
display('The Lambda standard errors from state 2 onwards are')
display(Lambdase)
display('The intercept and slope parameters are')
display(reshape(gamma(1:z,1),K,z/K))
display('The intercept and slope standard errors are')
display(reshape(gamma(1:z,2),K,z/K))

if slope == 'y' && intercept == 'y'
    betahat = Betahat;
end
[residuals,Qh_pval,Qh_mod_pval] = Portmanteau_test(slope,intercept,bold_y,Xbar,Zbar,betahat,xiT,K,T,p,M);
display('The Portmanteau and modified Portmanteau test p-values are')
display([Qh_pval Qh_mod_pval])

end

function y = objfun(pars,T,U,XIhat,That,K,M,intercept,slope)
% This is the log-likelihood function for calculating the restricted
% parameters of the covariance matrices.

B = reshape(pars(1:K^2),K,K);
Lambda = zeros(K,K,M);
A = Lambda;
invSigma = Lambda;
Lambda(:,:,1) = eye(K);
for m = 2:M
    Lambda(:,:,m) = diag(pars(K^2+1+K*(m-2):K^2+K*(m-1)));
end

for m = 1:M
    if slope == 'n' && intercept == 'n'
        A(:,:,m) = U*XIhat(:,:,m)*U';
    else
        A(:,:,m) = U(:,:,m)*XIhat(:,:,m)*U(:,:,m)';
    end
    invSigma(:,:,m) = inv(B*Lambda(:,:,m)*B');
end

y = (T)*log(abs(det(B)))+(1/2)*trace(invSigma(:,:,1)*A(:,:,1));

for m = 2:M
    y = y + (1/2)*That(m,:)*log(det(Lambda(:,:,m))) + (1/2)*trace(invSigma(:,:,m)*A(:,:,m));
end

end

function L = Lik(gamma,K,bold_y,Zbar,Xbar,xi1,T,M,p,z,intercept,slope,short_restr)
% This is the log-likelihood function of the model including intercept,
% slope, covariance and transition probabilities parameters.

% Get the parameter values from the gamma vector.
if slope == 'y' && intercept == 'y'
    Betahat = reshape(gamma(1:z),z/M,M);
else
    betahat = gamma(1:z);
end
if size(short_restr,1) == 0
    B = reshape(gamma(1 + z : z + K^2),K,K);
    sr = 0;
else
    B = zeros(K,K);
    sum_restr = sum(short_restr)';
    find_pars = find(sum_restr == 0);
    pars(find_pars) = 1; %#ok<*FNDSB>
    pars = pars';
    jj = 1;
    for ii = 1:K^2
        if pars(ii) == 1
            B(ii) = gamma(jj + z);
            jj = jj + 1;
        end
    end
    sr = size(short_restr,1);
end
P1 = reshape(gamma(1 + z + K^2 - sr:(M-1)^2 + z + K^2 - sr),M-1,M-1);
P1(:,M) = 1 - sum(P1(:,1:M-1),2);
P1(M,2:M) = gamma(1 + (M-1)^2 + z + K^2 - sr: M*(M-1) + z + K^2 - sr)';
P1(M,1) = 1 - sum(P1(M,2:M));
Lambda = zeros(K,K,M);
Lambda(:,:,1) = eye(K);
for m = 2:M
    Lambda(:,:,m) = diag(gamma(1 + M*(M-1) + z + K^2 + K*(m-2) - sr:M*(M-1) + z + K^2 + K*(m-1) - sr));
end

% Define the residuals matrix.
if slope == 'n'
    if intercept == 'n' % MSH(M)-SVAR(p) Model
        U = bold_y-kron(Xbar,eye(K))*betahat;
        U = reshape(U,K,T);
    else % MSIH(M)-SVAR(p) Model
        XXbar = Xbar(:,M:end,M);
        U = zeros(1,K*T,M);
        for m = 1:M
            U(:,:,m) = bold_y - kron(XXbar,eye(K))*[betahat(1+K*(m-1):K*m,1); betahat(K*M+1:end,1)];
        end
        U = reshape(U,K,T,M);
    end
else
    if intercept == 'n' % MSAH(M)-SVAR(p) Model
        Y = reshape(bold_y,K,T);
        U = zeros(K,T,M);
        for m = 1:M
            U(:,:,m) = Y - reshape(betahat(1+K+K^2*p*(m-1):K+K^2*p*m),K,K*p)*Zbar' - kron(ones(T,1)',betahat(1:K));
        end
    else % MSIAH(M)-SVAR(p) Model
        U = zeros(1,K*T,M);
        for m = 1:M
            U(:,:,m) = bold_y-kron(Xbar,eye(K))*Betahat(:,m);
        end
        U = reshape(U,K,T,M);
    end
end

% Main function starts here.
xit = zeros(M,T+1);
eta = zeros(M,T);
xit(:,1) = xi1;

L = zeros(1,T);

invSigma = zeros(K,K,M);
sqdetSigma = zeros(M,1);
for m = 1:M
    invSigma(:,:,m) = inv(B*Lambda(:,:,m)*B');
    sqdetSigma(m,:) = sqrt(det(B*Lambda(:,:,m)*B'));
end

if intercept == 'n'
    for t = 1:T
        for m = 1:M
            eta(m,t) = exp(-U(:,t)'*invSigma(:,:,m)*U(:,t)/2)/((2*pi)^(K/2)*sqdetSigma(m,:));
        end
    end
else
    for t = 1:T
        for m = 1:M
            eta(m,t) = exp(-U(:,t,m)'*invSigma(:,:,m)*U(:,t,m)/2)/((2*pi)^(K/2)*sqdetSigma(m,:));
        end
    end
end
for t = 1:T
    j  = t+1;
    xit(:,j) = ((P1'*xit(:,t)).*eta(:,t))/(ones(1,M)*((P1'*xit(:,t)).*eta(:,t)));
    L(1,t) = -log((P1'*xit(:,t))'*eta(:,t));
    L = sum(L,2);
end

end

function RecessionPlot(dates,Recessions)
% Description:
%
%   Overlay shaded bands to identify recessions on a time series plot.
%
% Input Arguments:
%
%   dates - Date numbers used in time series plot.
%
%   Recessions - numRecessions x 2 array of date numbers. Each row contains
%     the start and end date of a recession.

% Copyright 2010 The MathWorks, Inc.
% $Revision: 1.1.6.1 $   $Date: 2009/09/23 14:00:52 $

for r = 1:size(Recessions,1)
    
 if Recessions(r,2) > min(dates) && Recessions(r,1) <= max(dates)
        
  ph(r) = patch([Recessions(r,1) Recessions(r,1) Recessions(r,2) Recessions(r,2)], ...
             [get(gca,'YLim') fliplr(get(gca,'YLim'))], [0 0 0 0], 'k');
  set(ph(r),'FaceColor','r','FaceAlpha',0.4,'EdgeColor','none');
        
 end
    
end

end

function [residuals,Qh_pval,Qh_mod_pval] = Portmanteau_test(slope,intercept,bold_y,Xbar,Zbar,betahat,xiT,K,T,p,M)
% Function obtaining the residuals of the MS models in order to do a
% Portmanteau test.
if slope == 'n'
    if intercept == 'n' % MSH(M)-SVAR(p) Model
        residuals = bold_y-kron(Xbar,eye(K))*betahat;
        residuals = reshape(residuals,K,T);
    else % MSIH(M)-SVAR(p) Model
        residuals = bold_y - kron(Zbar,eye(K))*betahat(K*M+1:end,1);
        for m = 1:M
            residuals = residuals - kron(xiT(m,:)',eye(K))*betahat(1+K*(m-1):K*m,1);
        end
        residuals = reshape(residuals,K,T);
    end
else
    if intercept == 'n' % MSAH(M)-SVAR(p) Model
        residuals = bold_y - kron(ones(T,1),eye(K))*betahat(1:K);
        for m = 1:M
            residuals = residuals - kron(diag(xiT(m,:))*Zbar,eye(K))*betahat(1+K+K^2*p*(m-1):K+K^2*p*m);
        end
        residuals = reshape(residuals,K,T);
    else % MSIAH(M)-SVAR(p) Model
        residuals = zeros(K*T,1);
        for m = 1:M
            residuals = residuals - kron(diag(xiT(m,:))*Xbar,eye(K))*betahat(:,m);
        end
        residuals = bold_y + residuals;
        residuals = reshape(residuals,K,T);
    end
end

% Portmanteau test follows Ltkepohl p.169 - 171
h = 16;
C = zeros(K,K,h); C0 = 0;
for i = 1:h
    for t = i+1:T
        C(:,:,i) = C(:,:,i) + residuals(:,t)*residuals(:,t-i)'/T;
    end
end
for t = 1:T
    C0 = C0 + residuals(:,t)*residuals(:,t)'/T;
end

Qh = 0; Qh_mod = 0;
for i = 1:h
    Qh = Qh + C(:,:,i)'/(C0)*C(:,:,i)/(C0);
    Qh_mod = Qh_mod + C(:,:,i)'/(C0)*C(:,:,i)/(C0)/(T-i);
end
Qh = trace(Qh)*T; Qh_mod = trace(Qh_mod)*T^2;
Qh_pval = 1-chi2cdf(Qh,K^2*(h-p));
Qh_mod_pval = 1-chi2cdf(Qh_mod,K^2*(h-p));

end