% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [K,T,loglike_ms,gamma] = mssvar_unrestricted_B(p,M,y,range,intercept,slope)
% This function estimates the (restricted) MS-SVAR model parameters and their
% standard errors. The model being estimated is:
% y_t = nu1(s_t) + A_1(s_t)*y_t-1 + A_2(s_t)*y_t-2 + ... + A_p(s_t)*y_t-p + u_t,
% such that y_t is a stationary process and u_t ~
% N(0,Sigma(s_t)), according to a
% stochastic state s_t goverened by a first-order Markov process. The
% covaraince matrix always switches. All other parameters do so optionally.
% The errors can be decomposed as u_t = B*eps_t, where B is the
% short-run, (contemporaneous) matrix. Here the B matrix is
% assumed not to be fixed across states. This function is used to get the
% log likelihood to test whether the assumption of a state invariant B
% matrix is correct.

% MODEL DESCRIPTION:
% - MSH(M)-SVAR(p): switching covariance matrix
% - MSIH(M)-SVAR(p): switching covariance matrix and intercept term
% - MSAH(M)-SVAR(p): switching covariance matrix and AR coefficients
% - MSIAH(M)-SVAR(p): switching covariance matrix, intercept term and AR coefficients
% For the intercept term to switch, intercept = 'y', for the AR coeffiecints
% to switch, slope = 'y'. IMPORTANT: use 'n' otherwise!

% INPUTS:
% p - number of lags
% M - number of Markov states
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables (output)
    % T: number of observations not taken as given (output)
% range - (1x(T+p)) vector of dates used for horizontal axis in smoothed probabilities plot. If range = 0 no dates plotted.
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise.
% slope: slope = 'y' for a switching AR matrices, slope = 'n' otherwise.

% OUTPUTS:
% K - number of endogenous variables
% T - number of observations  not taken as given
% loglike_ms - log-likelihood of the Markov switching (MS) SVAR model
% gamma - matrix with column of parameter estimates and colum of their respective standard errors (see code for details).

%% Set up exogenous matrices

[Y,K,T,Zbar,Xbar,bold_y,z] = initialization(y,p,M,intercept,slope);

%% Initialize storing matrices

storeloglike = zeros(1,1);
KsiT = zeros(M,T,1);
Gamma = zeros(z + M*(M-1),1,1);
COV = zeros(K,K,M,1);
PP = zeros(M,M,1);

for o = 1:100
    o %#ok<NOPRT>
    
    %% Initial Values
    % beta is a vector of intercept and vectorized slope parameters
    if slope == 'n'
        if intercept == 'n' % MSH(M)-SVAR(p) Model
            beta = kron(Xbar'*Xbar,eye(K))\kron(Xbar',eye(K))*bold_y; % replaced inv(A)*b by A\b
            U = bold_y - kron(Xbar,eye(K))*beta;
            U = reshape(U,K,T);
            B = sqrtm(U*U'/T) + randn(K,K)/100; % Starting value for the B matrix
        else % MSIH(M)-SVAR(p) Model
            XX = Xbar(:,M:end,M);
            
            beta = (XX'*XX)\XX'*Y';
            
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = Y - beta'*XX';
            end
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
        end
    else
        if intercept == 'n' % MSAH(M)-SVAR(p) Model
            XX = Xbar(:,1:1+K*p,1);
            
            beta = (XX'*XX)\XX'*Y';
            
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = Y - beta'*XX';
            end
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
        else % MSIAH(M)-SVAR(p) Model
            Beta = zeros(K*(K*p+1),M);
            U = zeros(K*T,M);
            for m = 1:M
                Beta(:,m) = kron(Xbar'*Xbar,eye(K))\kron(Xbar',eye(K))*bold_y;
                U(:,m) = bold_y-kron(Xbar,eye(K))*Beta(:,m);
            end
            U = reshape(U,K,T,M);
            
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/1000;
        end
    end
    
    Lambda = zeros(K,K,M);
    Lambda(:,:,1) = eye(K); % Starting value for the Lambda matrix
    for m = 2:M
        Lambda(:,:,m) = o^(m-1)*eye(K);
    end
    
    % Matrix estimated as in the original model, however B and Lambda do
    % not play a role later on.
    Sigma = zeros(K,K,M);
    for m = 1:M
        Sigma(:,:,m) = B*Lambda(:,:,m)*B';
    end
    
    P1 = ones(M,1)*ones(1,M)/M; % Starting value of the transition probabilities. Note
    % probabilities add up to 1 row-wise.
    xi1 = ones(M,1)/M; % Starting value of the initial xi vector
    
    % Store the log-likelihood values per iteration for convergence criteria.
    Loglikelihood = zeros(1,1);
    
    % Exit condition, to be used in case of singulariites
    exit = 0;
    %% Main Program using the EM algorithm
    n = 1 %#ok<NOPRT>
    conv = 1;
    while conv>10^-6
        %% Expectation Step
        
        [loglikelihood,xiT,P2] = expectation_mssvar(T,K,M,Sigma,U,xi1,P1,intercept,slope);
        
        %% Maximization Step
        
        % Define the matrices, these are from the expectation step and stay
        % outside of the maximization loop.
        
        XIhat = zeros(T,T,M);
        for m = 1:M
            XIhat(:,:,m) = diag(xiT(m,2:end));
        end
        
        That = sum(xiT(:,2:T+1),2);
        
        %% Estimate the Covariance Matrices and the remaining Parameters
        
        % Estimate the intercept and slope parameters according to
        % Table 9.12 p.192 of Krolzig(1997)
        A = zeros(z,z);
        AA = zeros(z,K*T);
        Sigmahat = zeros(K,K,M);
        if slope == 'n'
            if intercept == 'n' % MSH(M)-SVAR(p) Model
                for m = 1:M
                    A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigma(:,:,m)));
                    AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigma(:,:,m)));
                end
                betahat = A\AA*bold_y;
                U_hat = bold_y-kron(Xbar,eye(K))*betahat;% The residual matrix.
                U_hat = reshape(U_hat,K,T);
                
                for m = 1:M
                    Sigmahat(:,:,m) = U_hat*XIhat(:,:,m)*U_hat'/That(m);
                end
            else % MSIH(M)-SVAR(p) Model
                for m = 1:M
                    A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigma(:,:,m)));
                    AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigma(:,:,m)));
                end
                betahat = A\AA*bold_y;
                U_hat = zeros(1,K*T,M); % Residual matrices with switching intercept term
                for m = 1:M
                    U_hat(:,:,m) = bold_y - kron(XX,eye(K))*[betahat(1+K*(m-1):K*m,1); betahat(K*M+1:end,1)];
                end
                U_hat = reshape(U_hat,K,T,M);
                
                for m = 1:M
                    Sigmahat(:,:,m) = U_hat(:,:,m)*XIhat(:,:,m)*U_hat(:,:,m)'/That(m);
                end
            end
        else % MSAH(M)-SVAR(p) Model
            if intercept == 'n'
                for m = 1:M
                    A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigma(:,:,m)));
                    AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigma(:,:,m)));
                end
                betahat = A\AA*bold_y;
                U_hat = zeros(K,T,M);
                for m = 1:M
                    U_hat(:,:,m) = Y - reshape(betahat(1+K+K^2*p*(m-1):K+K^2*p*m),K,K*p)*Zbar' - kron(ones(T,1)',betahat(1:K));
                end
                
                for m = 1:M
                    Sigmahat(:,:,m) = U_hat(:,:,m)*XIhat(:,:,m)*U_hat(:,:,m)'/That(m);
                end
            else % MSIAH(M)-SVAR(p) Model
                Betahat = zeros(z/M,M);
                for m = 1:M
                    Betahat(:,m) = kron((Xbar'*XIhat(:,:,m)*Xbar)\Xbar'*XIhat(:,:,m),eye(K))*bold_y;
                end
                U_hat = zeros(1,K*T,M);
                for m = 1:M
                    U_hat(:,:,m) = bold_y-kron(Xbar,eye(K))*Betahat(:,m);
                end
                U_hat = reshape(U_hat,K,T,M);
                
                for m = 1:M
                    Sigmahat(:,:,m) = U_hat(:,:,m)*XIhat(:,:,m)*U_hat(:,:,m)'/That(m);
                end
            end
        end
        
        % Check the eigenvalues of the covariance matrices. If smaller
        % than a certain number exit and start from the beginning.
        not_number = isnan(Sigmahat);
        infinite = isinf(Sigmahat);
        not_number = sum(not_number(:));
        infinite = sum(infinite(:));
        if not_number + infinite > 0
            exit = 1;
        else
            Eigenvalues_Sigmahat = zeros(K,1,M);
            for m = 1:M
                Eigenvalues_Sigmahat(:,1,m) = eig(Sigmahat(:,:,m));
            end
            low_eigenvalues = Eigenvalues_Sigmahat < 0.0000001;
            low_eigenvalues = sum(low_eigenvalues(:));
            if low_eigenvalues > 0
                exit = 1;
            end
        end
        
        %% Store new estimates of Expectation and Maximization step and repeat iteration
        % This is done outside the inner loop.
        
        U = U_hat;
        Sigma = Sigmahat;
        P1 = P2;
        xi1 = xiT(:,1);
        
        % Store estimated values of each iteration.
        Loglikelihood(n+1,1) = loglikelihood;
        
        % Convergence crieteria for main loop.
        conv = abs(Loglikelihood(n+1,1)-Loglikelihood(n,1));
        
        if exit == 1 || n >= 1000
            conv = 0;
        end
        
        n = n + 1 %#ok<NOPRT>
    end
    
    % Store the maximum log-likelihood and parameter estimates
    % from the current round of iterations on the outer loop.
    if exit == 1
        storeloglike(o,1) = -inf;
    else
        storeloglike(o,1) = loglikelihood;
    end
    KsiT(:,:,o) = xiT(:,2:T+1);
    COV(:,:,:,o) = Sigma;
    PP(:,:,o) = P1;
    
    Ppars = reshape(P1(1:M-1,1:M-1),(M-1)^2,1);
    Ppars(1+(M-1)^2:(M-1)^2+M-1,:) = reshape(P1(M,2:M),M-1,1);
    
    if slope == 'y' && intercept == 'y'
        Gamma(:,1,o) = [Betahat(:); Ppars(:)];
    else
        Gamma(:,1,o) = [betahat; Ppars(:)];
    end
    
end

% Get the maximum log-likelihood from all iterations on the outer
% loop and conseqently the optimal parameter values from that iteration.
storeloglike(storeloglike==0)=-inf;
[loglike_ms, I] = max(storeloglike);
xiT = KsiT(:,:,I);
Sigma = COV(:,:,:,I);
P = PP(:,:,I);
gamma = Gamma(:,1,I);

%% Plots and Display
% Define recessions according to NBER:
Recessions = [ datenum('15-May-1937'), datenum('15-Jun-1938');
    datenum('15-Feb-1945'), datenum('15-Oct-1945');
    datenum('15-Nov-1948'), datenum('15-Oct-1949');
    datenum('15-Jul-1953'), datenum('15-May-1954');
    datenum('15-Aug-1957'), datenum('15-Apr-1958');
    datenum('15-Apr-1960'), datenum('15-Feb-1961');
    datenum('15-Dec-1969'), datenum('15-Nov-1970');
    datenum('15-Nov-1973'), datenum('15-Mar-1975');
    datenum('15-Jan-1980'), datenum('15-Jul-1980');
    datenum('15-Jul-1981'), datenum('15-Nov-1982');
    datenum('15-Jul-1990'), datenum('15-Mar-1991');
    datenum('15-Mar-2001'), datenum('15-Nov-2001');
    datenum('15-Dec-2007'), datenum('15-Jun-2009') ];
Recessions = busdate(Recessions); % Until the next business day

% Plot probability of the states (and the log-likelihood)

if range == 0
    figure('Name','Smooth probabilities','NumberTitle','off')
    for m = 1:M
        subplot(M,1,m)
        plot(xiT(m,1:T),'LineWidth',2); axis tight;
        ylim([0 1])
    end
else
    range = range(:,p+1:end)';
    range = datenum([range,ones(T,2)]);
    figure('Name','Smooth probabilities','NumberTitle','off')
    for m = 1:M
        subplot(M,1,m)
        plot(range,xiT(m,1:T),'LineWidth',2);
        RecessionPlot(range, Recessions)
        set(gca,'XTick',range(1:20:end)')
        datetick('x','yyyy','keepticks')
        axis tight;
        ylim([0 1])
    end
end

% Display results
display('The number of free parameters are')
n=z + 0.5*M*K*(K+1) + M*(M-1);
display(n)
display('The maximum log-likelihood is')
fprintf('%10.6f',loglike_ms)
fprintf('\n')
display('The AIC value is')
fprintf('%10.6f ',-2*(loglike_ms-n));
fprintf('\n')
display('The SIC value is')
fprintf('%10.6f',-2*loglike_ms+log(T)*n);
fprintf('\n')
display('The covariance matrices are')
display(Sigma)
display('The tansition probabilities are')
display(P)
display('The intercept and slope parameters are')
display(reshape(gamma(1:z,1),K,z/K))
end

function RecessionPlot(dates,Recessions)
% Description:
%
%   Overlay shaded bands to identify recessions on a time series plot.
%
% Input Arguments:
%
%   dates - Date numbers used in time series plot.
%
%   Recessions - numRecessions x 2 array of date numbers. Each row contains
%     the start and end date of a recession.

% Copyright 2010 The MathWorks, Inc.
% $Revision: 1.1.6.1 $   $Date: 2009/09/23 14:00:52 $

for r = 1:size(Recessions,1)
    
    if Recessions(r,2) > min(dates) && Recessions(r,1) <= max(dates)
        
        ph(r) = patch([Recessions(r,1) Recessions(r,1) Recessions(r,2) Recessions(r,2)], ...
            [get(gca,'YLim') fliplr(get(gca,'YLim'))], [0 0 0 0], 'k'); %#ok<AGROW>
        set(ph(r),'FaceColor','r','FaceAlpha',0.4,'EdgeColor','none');
        
    end
    
end

end