% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [loglike_ms,omega,cov] = mssvecm(y,r,p,T,K,M,alpha,beta,long_restr,short_restr,Xi_tilde,range,trend,intercept,beta_est)
% This function estimates the (restricted) MS-SVEC model parameters and their
% standard errors. The model being estimated is:
% Delta y_t = nu1 (or nu1(s_t)) + (nu2*t +) Pi*y_t-1 + Gamma_1*Delta y_t-1 + ... + Gamma_p-1*Delta y_t-p+1 + u_t,
% where Pi = -alpha*beta' and u_t ~ N(0,Sigma(s_t)), according to a
% stochastic state s_t goverened by a first-order Markov process. The
% covaraince matrix always switches, the intercept term does so
% optionally. Errors are decomposed as u_t = B*eps_t, where B is the
% short-run, (contemporaneous) matrix and restrictions can be imposed on
% it, given by short_restr, and/or long-run restrictions on Xi_tilde*B, given by long_restr.

% Inputs
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables (output of svecm.m)
    % T: number of observations not taken as given (output of svecm.m)
% r: cointegration rank
% p: number of lags as in the VAR form of the VECM
% M: number of Markov states
% alpha - (Kxr) loadings matrix (output of svecm.m)
% beta - (Kxr) matrix of cointegration relationships (output of svecm.m)
% long_restr: matrix with long-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere
% short_restr: same as long_restr for restrictions directly on B
% Xi_tilde - (KxK) matrix from Granger's representation theorem (output of svecm.m)
% range - (1x(T+p)) vector of dates used for horizontal axis in smoothed probabilities plot. If range = 0 no dates plotted.
% trend: if trend = 'y', D_t = nu1 + nu2*t, otherwise D_t = nu1 (see svecm.m). Note, the trend term does not switch.
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise. Note, there is always an intercept term in the model.
% beta_est: if beta_est = 'y' the beta matriy is estimated in each optimization round, if beta_est = 'n' the original beta matrix from svecm.m is kept throughout.

% Outputs
% loglike_ms: log-likelihood of the Markov switching (MS) SVEC model
% omega - matrix with column of parameter estimates and colum of their respective standard errors (see code for details).
% cov - covariance matrix of model parameters. Dimension depends on the type of model specified.

%% Set up exogenous matrices

Deltay = diff(y,1,2);
DeltaY = Deltay(:,p:end);
Yminus1 = y(:,p:end-1);
bold_y = reshape(DeltaY,T*K,1);
[Xbar,z] = xbar(r,p,T,K,M,beta,Yminus1,Deltay,intercept,trend,beta_est);

%% Set up storing matrices

% Store the log-likelihood
storeloglike = -inf(1,1);
% Store the smoothed probabilities
KsiT = zeros(M,T,1);
% Store the covariance matrices
COV = zeros(K,K,M,1);
% Store the transition probabilities
PP = zeros(M,M,1);

% Store the intercept(s), alpha, Gamma, (trend), B, P and Lambda
% matrix parameters:
Omega = zeros(z + K^2 - size(short_restr,1) + M*(M-1) + K*(M-1),1,1);

for o = 1:100
    o %#ok<NOPRT>
    
    %% Initial Values
    % theta is a vector of constant terms, (nu1(s_t) - (M*Kx1) (and nu2) - (Kx1)), the loading matrix,
    % (alpha - (Kxr)) and the parameter matrices, (Gamma - (KxK*(p-1)))
    
    if beta_est == 'n'
        beta_orth = null(beta','r');
        if intercept == 'n'
            theta = (kron(Xbar'*Xbar,eye(K)))\kron(Xbar',eye(K))*bold_y; % Replaced inv(kron(Xbar'*Xbar,eye(K)))*
            U = bold_y-kron(Xbar,eye(K))*theta;
            U = reshape(U,K,T); % (KxT) matrix of residuals
            B = sqrtm(U*U'/T) + randn(K,K)/100;
        else % model with switching intercept
            A = 0; AA = 0;
            for m = 1:M
                A = A + kron(Xbar(:,:,m)'*Xbar(:,:,m),eye(K));
                AA = AA + kron(Xbar(:,:,m)',eye(K));
            end
            theta = A\AA*bold_y; % replaced inv(A)*
            
            XXbar = Xbar(:,M:end,M);
            U = zeros(1,K*T,M);
            for m = 1:M
                U(:,:,m) = bold_y - kron(XXbar,eye(K))*[theta(1+K*(m-1):K*m,1); theta(K*M+1:end,1)];
            end
            U = reshape(U,K,T,M);
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/100;
        end
    else % beta estimated by RR regression
        Pi = -alpha*beta';
        if intercept == 'n'
            theta = (kron(Xbar'*Xbar,eye(K)))\kron(Xbar',eye(K))*bold_y;
            theta = reshape(theta,K,(size(theta,1))/K);
            U = DeltaY - Pi*Yminus1 - theta*Xbar';
            B = sqrtm(U*U'/T) + randn(K,K)/100;
        else % model with switching intercept
            XXbar = Xbar(:,M:end,M);
            theta = (kron(XXbar'*XXbar,eye(K)))\kron(XXbar',eye(K))*bold_y;
            theta = reshape(theta,K,(size(theta,1))/K);
            U = zeros(K,T,M);
            for m = 1:M
                U(:,:,m) = DeltaY - Pi*Yminus1 - theta*XXbar';
            end
            B = sqrtm(U(:,:,1)*U(:,:,1)'/T) + randn(K,K)/100;
        end
    end
    
    % Starting values for the Lambda matrix
    Lambda = zeros(K,K,M);
    Lambda(:,:,1) = eye(K);
    for m = 2:M
        Lambda(:,:,m) = o^(m-1)*eye(K);
    end
    
    % Covariance matrices
    Sigma = zeros(K,K,M);
    for m = 1:M
        Sigma(:,:,m) = B*Lambda(:,:,m)*B';
    end
    
    % Starting value of the transition probabilities. Note probabilities add up to 1 row-wise.
    P1 = ones(M,1)*ones(1,M)/M;
    % Starting value of the initial xit_0|0 vector
    xi1 = ones(M,1)/M;
    
    % Store the log-likelihood values:
    Loglikelihood = zeros(1,1);
    
    %% Main Program using the EM algorithm
    n = 1 %#ok<NOPRT>
    conv1 = 1;
    while conv1 > 10^-6
        %% Expectation Step
        
        [loglikelihood,xiT,P2] = expectation_mssvecm(T,K,M,Sigma,U,xi1,P1,intercept);
        %% Maximization Step
        
        % Estimation of the Covariance Matrices and the remaining Parameters.
        % This is a loop where first the B and Lambda matrix parameters are
        % estimated through the function objfun and then these values are
        % used to estimate the remaining parameters. From these, the new
        % residual matrix, U is estimated and this is plugged back into the
        % function objfun. For the restricted model both short and long-run
        % restrictions are used and the long-run restrictions are updated.
        
        % Define the matrices, these are from the expectation step and stay
        % outside of the maximization loop.
        XIhat = zeros(T,T,M);
        for m = 1:M
            XIhat(:,:,m) = diag(xiT(m,2:end));
        end
        
        That = sum(xiT(:,2:T+1),2);
        
        % Lower bound for the Lambda parameters for fmincon
        lb = -inf(K^2 + K*(M-1),1);
        lb(end-K*(M-1)+1:end,1) = 0.001;
        
        % Options displaying results.
        options = optimset('display','final','Algorithm', 'interior-point','MaxFunEvals', 20000, ...
            'MaxIter', 20000, 'DerivativeCheck','on','Diagnostics','off','GradObj','off','LargeScale','off');
        
        j = 1;
        Fval = zeros(1,1); % Matrix storing the maximum values of objfun.
        Pars = zeros(K^2 + K*(M-1),1); % Stores the parameter values.
        conv2 = 1;
        while conv2 > 10^-6
            % Starting value of the parameters to be estimated using a non-linear
            % solver, fmincon. Parameters are stored in a vector pars0 and
            % are updated at each iteration.
            Lambdapars = zeros(K*(M-1),1);
            for m = 2:M
                Lambdapars(1+K*(m-2):K*(m-1),:) = diag(Lambda(:,:,m));
            end
            
            pars0 = [B(:); Lambdapars];
            
            % Estimate the B and Lambda parameters.
            if size(short_restr,1) + size(long_restr,1) == 0 % unrestricted case:
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept), pars0,[],[],[],[],lb,[],[],options);
            else % restricted case according to restriction:
                if size(long_restr,1) == 0
                    aeq = short_restr;
                elseif size(short_restr,1) == 0
                    long_restr_updated = long_restr*kron(eye(K),Xi_tilde);
                    aeq = long_restr_updated;
                else
                    long_restr_updated = long_restr*kron(eye(K),Xi_tilde);
                    aeq = [long_restr_updated; short_restr];
                end
                Aeq = [aeq zeros(size(aeq,1),K*(M-1))];
                beq = zeros(size(Aeq,1),1);
                [pars,fval] = fmincon(@(pars) objfun(pars,T,U,XIhat,That,K,M,intercept), pars0,[],[],Aeq,beq,lb,[],[],options);
            end
            
            Fval(j+1,1) = fval;
            Pars(:,j) = pars;
            
            % Return the parameter estimates.
            Bhat = reshape(pars(1:K^2),K,K);
            Lambdahat = zeros(K,K,M);
            Lambdahat(:,:,1) = eye(K);
            for m = 2:M
                Lambdahat(:,:,m) = diag(pars(K^2+1+K*(m-2):K^2+K*(m-1)));
            end
            
            Sigmahat = zeros(K,K,M);
            for m = 1:M
                Sigmahat(:,:,m) = Bhat*Lambdahat(:,:,m)*Bhat';
            end
            
            exit = singlularity_check(Sigmahat,K,M);
            
            % Estimate the remaining parameters. This is done according to
            % model specification
            A = 0; AA = 0;
            if beta_est == 'n'
                if intercept == 'n'
                    for m = 1:M
                        A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    
                    thetahat = A\AA*bold_y;
                    U_hat = bold_y-kron(Xbar,eye(K))*thetahat;
                    U_hat = reshape(U_hat,K,T);
                    
                    if size(long_restr,1) > 0 % formulate the long-run impact matrix, Xi_tilde
                        if trend == 'n'
                            alpha = reshape(thetahat(K + 1:K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(1+r) + 1:K*(1+r) + K^2*(p-1),1),K,K*(p-1));
                        else
                            alpha = reshape(thetahat(2*K + 1:2*K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(2+r) + 1:K*(2+r) + K^2*(p-1),1),K,K*(p-1));
                        end
                        
                        alpha_orth = null(alpha','r');
                        lagpolynomial = zeros(K,K);
                        for i = 1:p-1
                            lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
                        end
                        lagpolynomial = eye(K) + lagpolynomial;
                        Xi_tilde_hat = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';
                        Xi_tilde = Xi_tilde_hat;
                    end
                else % model with switching intercept
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    
                    thetahat = A\AA*bold_y;
                    U_hat = zeros(1,K*T,M);
                    for m = 1:M
                        U_hat(:,:,m) = bold_y - kron(XXbar,eye(K))*[thetahat(1+K*(m-1):K*m,1); thetahat(K*M+1:end,1)];
                    end
                    U_hat = reshape(U_hat,K,T,M);
                    
                    if size(long_restr,1) > 0 % formulate the long-run impact matrix, Xi_tilde
                        if trend == 'n'
                            alpha = reshape(thetahat(M*K + 1:M*K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(M+r) + 1:K*(M+r) + K^2*(p-1),1),K,K*(p-1));
                        else
                            alpha = reshape(thetahat((1+M)*K + 1:(1+M)*K + K*r,1),K,r);
                            Gamma = reshape(thetahat(K*(1+M+r) + 1:K*(1+M+r) + K^2*(p-1),1),K,K*(p-1));
                        end
                        
                        alpha_orth = null(alpha','r');
                        lagpolynomial = zeros(K,K);
                        for i = 1:p-1
                            lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
                        end
                        lagpolynomial = eye(K) + lagpolynomial;
                        Xi_tilde_hat = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';
                        Xi_tilde = Xi_tilde_hat;
                    end
                end
            else % beta estimated by RR regression
                if intercept == 'n'
                    for m = 1:M
                        A = A + kron(Xbar'*XIhat(:,:,m)*Xbar,inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    Pihat = A\AA*bold_y; Nhat = A\AA*reshape(Yminus1,T*K,1);
                    Pihat = reshape(Pihat,K,size(Pihat,1)/K); Nhat = reshape(Nhat,K,size(Nhat,1)/K);
                    Uhat = DeltaY - Pihat*Xbar'; Vhat = Yminus1 - Nhat*Xbar';
                else % model with switching intercept
                    for m = 1:M
                        A = A + kron(Xbar(:,:,m)'*XIhat(:,:,m)*Xbar(:,:,m),inv(Sigmahat(:,:,m)));
                        AA = AA + kron(Xbar(:,:,m)'*XIhat(:,:,m),inv(Sigmahat(:,:,m)));
                    end
                    Pihat = A\AA*bold_y; Nhat = A\AA*reshape(Yminus1,T*K,1);
                    Pihat = reshape(Pihat,K,size(Pihat,1)/K); Nhat = reshape(Nhat,K,size(Nhat,1)/K);
                    Uhat = zeros(K,T*M); Vhat = Uhat;
                    for m = 1:M
                        Uhat(:,1+(m-1)*T:m*T) = DeltaY - Pihat*Xbar(:,:,m)'; Vhat(:,1+(m-1)*T:m*T) = Yminus1 - Nhat*Xbar(:,:,m)';
                    end
                    
                end
                SIGMAvv = Vhat*Vhat'/T; SIGMAuu = Uhat*Uhat'/T; SIGMAuv = Uhat*Vhat'/T; SIGMAvu = SIGMAuv';
                % Check for stopping the algorithm:
                not_num = zeros(K,K,3); infin = not_num;
                not_num(:,:,1) = isnan(SIGMAvv); not_num(:,:,2) = isnan(SIGMAuu); not_num(:,:,3) = isnan(SIGMAuv);
                not_num = sum(not_num(:));
                infin(:,:,1) = isinf(SIGMAvv); infin(:,:,2) = isinf(SIGMAuu); infin(:,:,3) = isinf(SIGMAuv);
                infin = sum(infin(:));
                if not_num + infin > 0
                    exit = 1;
                    alpha = randn(K,r); Gamma = randn(K,K*(p-1)); beta_orth = randn(K,K-r); U_hat = randn(K,T); nu1 = randn(K,1); nu2 = nu1;
                else
                    Low_eigs = zeros(K,1,3);
                    Low_eigs(:,:,1) = eig(SIGMAvv); Low_eigs(:,:,2) = eig(SIGMAuu); Low_eigs(:,:,3) = abs(eig(SIGMAuv));
                    Low_eigs = Low_eigs < 0.00001;
                    Low_eigs = sum(Low_eigs(:));
                    if Low_eigs > 0
                        exit = 1;
                        alpha = randn(K,r); Gamma = randn(K,K*(p-1)); beta_orth = randn(K,K-r); U_hat = randn(K,T); nu1 = randn(K,1); nu2 = nu1;
                        % Stopping check done
                    else
                        [a lambda] = eigs(SIGMAvv\SIGMAvu/(SIGMAuu)*SIGMAuv,r); %#ok<NASGU>
                        beta_b = zeros(size(a));
                        for i = 1:size(a,2)
                            beta_b(:,i) = a(:,i)/(a(:,i)'*SIGMAvv*a(:,i))^0.5; % [20.2.11]
                        end
                        beta_r = beta_b(1:r,:); % Normalization as in Ltkepohl (2005)
                        beta = beta_b/beta_r; % the beta matrix
                        
                        alpha = -SIGMAuv*beta_b*beta_r'; % the alpha matrix
                        
                        Pi = -alpha*beta'; % estimated Pi matrix
                        
                        % Parameter matrices:
                        if intercept == 'n'
                            if trend == 'n'
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+2:K*i+1) - Pi*Nhat(:,K*(i-1)+2:K*i+1);
                                end
                                nu1 = Pihat(:,1) - Pi*Nhat(:,1); % constant term
                                
                                Gam = [nu1 Gamma];
                                U_hat = DeltaY - Pi*Yminus1 - Gam*Xbar';
                            else
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+3:K*i+2) - Pi*Nhat(:,K*(i-1)+3:K*i+2);
                                end
                                nu1 = Pihat(:,1) - Pi*Nhat(:,1); % constant term
                                nu2 = Pihat(:,2) - Pi*Nhat(:,2); % trend term
                                
                                Gam = [nu1 nu2 Gamma];
                                U_hat = DeltaY - Pi*Yminus1 - Gam*Xbar';
                            end
                        else % model with switching intercept
                            if trend == 'n'
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+M+1:K*i+M) - Pi*Nhat(:,K*(i-1)+M+1:K*i+M);
                                end
                                nu1 = Pihat(:,1:M) - Pi*Nhat(:,1:M); % constant terms
                                
                                Gam = [nu1 Gamma];
                                U_hat = zeros(K,T,M);
                                for m = 1:M
                                    U_hat(:,:,m) = DeltaY - Pi*Yminus1 - Gam*Xbar(:,:,m)';
                                end
                            else
                                for i = 1:p-1
                                    Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+M+2:K*i+M+1) - Pi*Nhat(:,K*(i-1)+M+2:K*i+M+1);
                                end
                                nu1 = Pihat(:,1:M) - Pi*Nhat(:,1:M); % constant terms
                                nu2 = Pihat(:,M+1) - Pi*Nhat(:,M+1); % trend term
                                
                                Gam = [nu1 nu2 Gamma];
                                U_hat = zeros(K,T,M);
                                for m = 1:M
                                    U_hat(:,:,m) = DeltaY - Pi*Yminus1 - Gam*Xbar(:,:,m)';
                                end
                            end
                        end
                        
                        % formulate the long-run impact matrix, Xi_tilde
                        if size(long_restr,1) > 0
                            alpha_orth = null(alpha','r');
                            beta_orth = null(beta','r');
                            lagpolynomial = zeros(K,K);
                            for i = 1:p-1
                                lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
                            end
                            lagpolynomial = eye(K) + lagpolynomial;
                            Xi_tilde_hat = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';
                            Xi_tilde = Xi_tilde_hat;
                        end
                    end
                end
            end
            
            % Store the updated matrices
            B = Bhat;
            Lambda = Lambdahat;
            U = U_hat;
            
            % Convergence criteria and maximum number of iterations.
            conv2 = -(Fval(j+1,1)-Fval(j,1));
            
            if exit == 1 || j >= 100
                conv2 = 0;
            end
            
            j = j + 1;
        end
        
        %% Store new estimates of Expectation and Maximization step and repeat iteration
        
        Sigma = Sigmahat;
        P1 = P2;
        xi1 = xiT(:,1);
        
        % Store estimated log-likelihood values of each iteration.
        Loglikelihood(n+1,1) = loglikelihood;
        
        % Convergence crieteria for main loop.
        conv1 = abs(Loglikelihood(n+1,1)-Loglikelihood(n,1));
        
        if exit == 1 || n >= 1000
            conv1 = 0;
        end
        
        n = n + 1 %#ok<NOPRT>
    end
    
    % Store the maximum log-likelihood and parameter estimates
    % from the current round of iterations on the outer loop.
    if exit == 1
        storeloglike(o,1) = -inf;
    else
        storeloglike(o,1) = loglikelihood;
    end
    KsiT(:,:,o) = xiT(:,2:T+1);
    COV(:,:,:,o) = Sigma;
    PP(:,:,o) = P1;
    
    vecB = B(:);
    if size(short_restr,1) == 1
        Bpars = vecB(find(short_restr' == 0));
    elseif size(short_restr,1) > 1
        sum_restr = sum(short_restr)';
        Bpars = vecB(find(sum_restr == 0));
    else
        Bpars = vecB;
    end
    
    Lambdapars = zeros(K*(M-1),1);
    for m = 2:M
        Lambdapars(1+K*(m-2):K*(m-1),:) = diag(Lambda(:,:,m));
    end
    
    Ppars = reshape(P1(1:M-1,1:M-1),(M-1)^2,1);
    Ppars(1+(M-1)^2:(M-1)^2+M-1,:) = reshape(P1(M,2:M),M-1,1);
    
    if beta_est == 'n'
        Omega(:,1,o) = [thetahat; Bpars; Ppars; Lambdapars];
    else
        betapars = beta(r+1:end,:);
        if trend == 'n'
            Omega(:,1,o) = [nu1(:); alpha(:); betapars(:); Gamma(:); Bpars; Ppars; Lambdapars];
        else
            Omega(:,1,o) = [nu1(:); nu2; alpha(:); betapars(:); Gamma(:); Bpars; Ppars; Lambdapars];
        end
    end
end

% Get the maximum log-likelihood from all iterations on the outer
% loop and conseqently the optimal parameter values from that iteration.
storeloglike(find(storeloglike==0))=-inf;
[loglike_ms, Index] = max(storeloglike);
xiT = KsiT(:,:,Index);
sigma = COV(:,:,:,Index);
P = PP(:,:,Index);
omega = Omega(:,:,Index);


%% Standard errors
% Once the EM algorithm has converged all parameter estimates are used in the
% log-likelihood function for the whole model, Lik. Standard
% errors are obtained from the (negative of the) inverse of the Hessian matrix at the
% optimum.

% Here hess is the hessian (and err is the error of approximation)
[hess] = hessian(@(omega) Lik(omega,K,bold_y,Xbar,xi1,T,M,z,p,short_restr,intercept,trend,beta_est,r,DeltaY,Yminus1),omega);

cov = inv(hess); % covariance matrix
sterr = zeros(size(omega)); % standard error matrix
for i = 1:size(omega,1)
    sterr(i,1) = sqrt(cov(i,i));
end

omega = [omega sterr];

%% Plots and Display
% Define recessions according to NBER:
Recessions = [ datenum('15-May-1937'), datenum('15-Jun-1938');
    datenum('15-Feb-1945'), datenum('15-Oct-1945');
    datenum('15-Nov-1948'), datenum('15-Oct-1949');
    datenum('15-Jul-1953'), datenum('15-May-1954');
    datenum('15-Aug-1957'), datenum('15-Apr-1958');
    datenum('15-Apr-1960'), datenum('15-Feb-1961');
    datenum('15-Dec-1969'), datenum('15-Nov-1970');
    datenum('15-Nov-1973'), datenum('15-Mar-1975');
    datenum('15-Jan-1980'), datenum('15-Jul-1980');
    datenum('15-Jul-1981'), datenum('15-Nov-1982');
    datenum('15-Jul-1990'), datenum('15-Mar-1991');
    datenum('15-Mar-2001'), datenum('15-Nov-2001');
    datenum('15-Dec-2007'), datenum('15-Jun-2009') ];
Recessions = busdate(Recessions); % Until the next business day

% Plot of smoothed probabilities:
if range == 0
    figure('Name','Smooth probabilities','NumberTitle','off')
    for m = 1:M
        subplot(M,1,m)
        plot(xiT(m,1:T),'LineWidth',2); axis tight;
        ylim([0 1])
    end
else
    range = range(:,p+1:end)';
    range = datenum([range,ones(T,2)]);
    figure('Name','Smooth probabilities','NumberTitle','off')
    for m = 1:M
        subplot(M,1,m)
        plot(range,xiT(m,1:T),'LineWidth',2);
        RecessionPlot(range, Recessions)
        set(gca,'XTick',range(1:20:end)')
        datetick('x','yyyy','keepticks')
        axis tight;
        ylim([0 1])
    end
end

% Define matrices to display:
if beta_est == 'n'
    theta = reshape(omega(1:z,1),K,z/K);
    thetase = reshape(omega(1:z,2),K,z/K);
else
    beta = eye(r); betase = zeros(r,r);
    if intercept == 'n'
        nu1 = omega(1:K,1); nu1se = omega(1:K,2);
        if trend == 'n'
            alpha = reshape(omega(1 +  K: K + K*r,1),K,r); alphase = reshape(omega(1 + K : K + K*r,2),K,r);
            beta = [beta; reshape(omega(1 + K*(1+r) : K*(1+r) + (K-r)*r,1),K-r,r)]; betase = [betase; reshape(omega(1 + K*(1+r) : K*(1+r) + (K-r)*r,2),K-r,r)];
            Gamma = reshape(omega(1 + K*(1+r) + (K-r)*r : z,1),K,K*(p-1)); Gammase = reshape(omega(1 + K*(1+r) + (K-r)*r : z,2),K,K*(p-1));
            theta = [nu1 alpha beta Gamma]; thetase = [nu1se alphase betase Gammase];
        else
            nu2 = omega(1 + K : 2*K,1); nu2se = omega(1 + K : 2*K,2);
            alpha = reshape(omega(1 + 2*K : 2*K + K*r,1),K,r); alphase = reshape(omega(1 + 2*K : 2*K + K*r,2),K,r);
            beta = [beta; reshape(omega(1 + K*(2+r) : K*(2+r) + (K-r)*r,1),K-r,r)]; betase = [betase; reshape(omega(1 + K*(2+r) : K*(2+r) + (K-r)*r,2),K-r,r)];
            Gamma = reshape(omega(1 + K*(2+r) + (K-r)*r : z,1),K,K*(p-1)); Gammase = reshape(omega(1 + K*(2+r) + (K-r)*r : z,2),K,K*(p-1));
            theta = [nu1 nu2 alpha beta Gamma]; thetase = [nu1se nu2se alphase betase Gammase];
        end
    else
        nu1 = reshape(omega(1:K*M,1),K,M); nu1se = reshape(omega(1:K*M,2),K,M);
        if trend == 'n'
            alpha = reshape(omega(1 +  K*M: K*M + K*r,1),K,r); alphase = reshape(omega(1 + K*M : K*M + K*r,2),K,r);
            beta = [beta; reshape(omega(1 + K*(M+r) : K*(M+r) + (K-r)*r,1),K-r,r)]; betase = [betase; reshape(omega(1 + K*(M+r) : K*(M+r) + (K-r)*r,2),K-r,r)];
            Gamma = reshape(omega(1 + K*(M+r) + (K-r)*r : z,1),K,K*(p-1)); Gammase = reshape(omega(1 + K*(M+r) + (K-r)*r : z,2),K,K*(p-1));
            theta = [nu1 alpha beta Gamma]; thetase = [nu1se alphase betase Gammase];
        else
            nu2 = omega(1 + K*M : K*(1+M),1); nu2se = omega(1 + K*M : K*(1+M),2);
            alpha = reshape(omega(1 + K*(1+M) : K*(1+M) + K*r,1),K,r); alphase = reshape(omega(1 + K*(1+M) : K*(1+M) + K*r,2),K,r);
            beta = [beta; reshape(omega(1 + K*(1+M+r) : K*(1+M+r) + (K-r)*r,1),K-r,r)]; betase = [betase; reshape(omega(1 + K*(1+M+r) : K*(1+M+r) + (K-r)*r,2),K-r,r)];
            Gamma = reshape(omega(1 + K*(1+M+r) + (K-r)*r : z,1),K,K*(p-1)); Gammase = reshape(omega(1 + K*(1+M+r) + (K-r)*r : z,2),K,K*(p-1));
            theta = [nu1 nu2 alpha beta Gamma]; thetase = [nu1se nu2se alphase betase Gammase];
        end
    end
end

if size(short_restr,1) == 0
    B = reshape(omega(1 + z : z + K^2,1),K,K);
    Bse = reshape(omega(1 + z : z + K^2,2),K,K);
else
    find_pars = find(sum_restr == 0);
    pars(find_pars) = 1; %#ok<*FNDSB>
    pars = pars';
    Bse = zeros(K,K);
    B = zeros(K,K);
    jj = 1;
    for ii = 1:K^2
        if pars(ii) == 1
            B(ii) = omega(jj + z,1);
            Bse(ii) = omega(jj + z,2);
            jj = jj + 1;
        end
    end
end

sr = size(short_restr,1);
Pse = reshape(omega(1 + z + K^2 - sr:(M-1)^2 + z + K^2 - sr,2),M-1,M-1);
Pse(M,2:M) = omega(1+(M-1)^2 + z + K^2 - sr: M*(M-1) + z + K^2 - sr,2)';

Lambda = reshape(omega(1 + M*(M-1) + z + K^2 - sr:M*(M-1) + z + K^2 + K*(M-1) - sr,1),K,M-1);
Lambdase = reshape(omega(1 + M*(M-1) + z + K^2 - sr:M*(M-1) + z + K^2 + K*(M-1) - sr,2),K,M-1);

% Display results
fprintf('The number of free parameters are')
fprintf('\n')
n=size(omega,1) - sr;
fprintf('%10.0f',n)
fprintf('\n')
fprintf('The maximum log-likelihood is')
fprintf('\n')
fprintf('%10.6f',loglike_ms)
fprintf('\n')
fprintf('The AIC value is')
fprintf('\n')
fprintf('%10.6f',-2*(loglike_ms-n))
fprintf('\n')
fprintf('The SIC value is')
fprintf('\n')
fprintf('%10.6f',-2*loglike_ms+log(T)*n)
fprintf('\n')
display('The B matrix is')
display(B)
display('The standard errors for the B matrix are')
display(Bse)
display('The covariance matrices are')
display(sigma*1000)
display('The tansition probabilities are')
display(P)
display('The tansition probabilities standard errors are')
display(Pse)
display('The Lambda parameters from state 2 onwards are')
display(Lambda)
display('The Lambda standard errors from state 2 onwards are')
display(Lambdase)
display('The intercept, alpha, (beta), slope (and trend) parameters are')
display(theta)
display('The intercept, alpha, (beta), slope (and trend) standard errors are')
display(thetase)

[residuals,Qh_pval,Qh_mod_pval] = Portmanteau_test(beta_est,omega(:,1),z,intercept,bold_y,Xbar,K,T,M,p,r,trend,xiT,DeltaY,Yminus1);
display('The Portmanteau and modified Portmanteau test p-values are')
display([Qh_pval Qh_mod_pval])


end

%% Functions start here

%% Objective function for the B and Lambda parameters
function y = objfun(pars,T,U,XIhat,That,K,M,intercept)
% This is the log-likelihood function for calculating the (unrestricted)
% parameters of the B and Lambda matrices.

B = reshape(pars(1:K^2),K,K);
Lambda = zeros(K,K,M);
A = Lambda;
invSigma = Lambda;
Lambda(:,:,1) = eye(K);
for m = 2:M
    Lambda(:,:,m) = diag(pars(1 + K^2 + K*(m-2):K^2 + K*(m-1)));
end

for m = 1:M
    if intercept == 'n'
        A(:,:,m) = U*XIhat(:,:,m)*U';
    else
        A(:,:,m) = U(:,:,m)*XIhat(:,:,m)*U(:,:,m)';
    end
    invSigma(:,:,m) = inv(B*Lambda(:,:,m)*B');
end

y = T*log(abs(det(B)))+(1/2)*trace(invSigma(:,:,1)*A(:,:,1));

for m = 2:M
    y = y + (1/2)*That(m,:)*log(det(Lambda(:,:,m))) + (1/2)*trace(invSigma(:,:,m)*A(:,:,m));
end

end

%% Singularity check of the Sigmahat matrix
function exit = singlularity_check(Sigmahat,K,M)

% Check the eigenvalues of the covariance matrices. If smaller
% than a certain number exit and start from the beginning.
exit = 0;
not_number = isnan(Sigmahat);
infinite = isinf(Sigmahat);
not_number = sum(not_number(:));
infinite = sum(infinite(:));
if not_number + infinite > 0
    exit = 1;
else
    Eigenvalues_Sigmahat = zeros(K,1,M);
    for m = 1:M
        Eigenvalues_Sigmahat(:,1,m) = eig(Sigmahat(:,:,m));
    end
    low_eigenvalues = Eigenvalues_Sigmahat < 0.00000001;
    low_eigenvalues = sum(low_eigenvalues(:));
    if low_eigenvalues > 0
        exit = 1;
    end
end

end

%% Log-likelihood function of all unrestricted model parameters
function L = Lik(omega,K,bold_y,Xbar,xi1,T,M,z,p,short_restr,intercept,trend,beta_est,r,DeltaY,Yminus1)
% This is the log-likelihood function of the model which includes
% all parameters estimated previously, contained in omega.

% Get the parameter values from the omega vector.
if beta_est == 'n'
    thetahat = omega(1:z);
    if intercept == 'n'
        U = bold_y-kron(Xbar,eye(K))*thetahat;
        U = reshape(U,K,T);
    else
        XXbar = Xbar(:,M:end,M);
        U = zeros(1,K*T,M);
        for m = 1:M
            U(:,:,m) = bold_y - kron(XXbar,eye(K))*[thetahat(1+K*(m-1):K*m,1); thetahat(K*M+1:end,1)];
        end
        U = reshape(U,K,T,M);
    end
else
    beta = eye(r);
    if intercept == 'n'
        nu1 = omega(1:K);
        if trend == 'n'
            alpha = reshape(omega(1 + K : K + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(1+r) : K*(1+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(1+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu1 Gamma];
        else
            nu2 = omega(1 + K : 2*K);
            alpha = reshape(omega(1 + 2*K : 2*K + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(2+r) : K*(2+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(2+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu1 nu2 Gamma];
        end
        Pi = -alpha*beta';
        U = DeltaY - Pi*Yminus1 - Gam*Xbar';
    else
        nu1 = reshape(omega(1:K*M),K,M);
        if trend == 'n'
            alpha = reshape(omega(1 + K*M : K*M + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(M+r) : K*(M+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(M+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu1 Gamma];
        else
            nu2 = omega(1 + K*M : K + K*M);
            alpha = reshape(omega(1 + K*(1+M) : K*(1+M) + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(1+M+r) : K*(1+M+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(1+M+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu1 nu2 Gamma];
        end
        Pi = -alpha*beta';
        U = zeros(K,T,M);
        for m = 1:M
            U(:,:,m) = DeltaY - Pi*Yminus1 - Gam*Xbar(:,:,m)';
        end
    end
end

if size(short_restr,1) == 0
    B = reshape(omega(1 + z : z + K^2),K,K);
    sr = 0;
else
    B = zeros(K,K);
    sum_restr = sum(short_restr)';
    find_pars = find(sum_restr == 0);
    pars(find_pars) = 1; %#ok<*FNDSB>
    pars = pars';
    jj = 1;
    for ii = 1:K^2
        if pars(ii) == 1
            B(ii) = omega(jj + z);
            jj = jj + 1;
        end
    end
    sr = size(short_restr,1);
end
P1 = reshape(omega(1 + z + K^2 - sr:(M-1)^2 + z + K^2 - sr),M-1,M-1);
P1(:,M) = 1 - sum(P1(:,1:M-1),2);
P1(M,2:M) = omega(1+(M-1)^2 + z + K^2 - sr: M*(M-1) + z + K^2 - sr)';
P1(M,1) = 1 - sum(P1(M,2:M));
Lambda = zeros(K,K,M);
Lambda(:,:,1) = eye(K);
for m = 2:M
    Lambda(:,:,m) = diag(omega(1 + M*(M-1) + z + K^2 + K*(m-2) - sr:M*(M-1) + z + K^2 + K*(m-1) - sr));
end

% Main function starts here.
xit = zeros(M,T+1);
eta = zeros(M,T);
xit(:,1) = xi1;

L = zeros(1,T);
invSigma = zeros(K,K,M);
sqdetSigma = zeros(M,1);
for m = 1:M
    invSigma(:,:,m) = inv(B*Lambda(:,:,m)*B');
    sqdetSigma(m,:) = sqrt(det(B*Lambda(:,:,m)*B'));
end

if intercept == 'n'
    for t = 1:T
        for m = 1:M
            eta(m,t) = exp(-U(:,t)'*invSigma(:,:,m)*U(:,t)/2)/((2*pi)^(K/2)*sqdetSigma(m,:));
        end
    end
else
    for t = 1:T
        for m = 1:M
            eta(m,t) = exp(-U(:,t,m)'*invSigma(:,:,m)*U(:,t,m)/2)/((2*pi)^(K/2)*sqdetSigma(m,:));
        end
    end
end
for t = 1:T
    j  = t+1;
    xit(:,j) = ((P1'*xit(:,t)).*eta(:,t))/(ones(1,M)*((P1'*xit(:,t)).*eta(:,t)));
    L(1,t) = -log((P1'*xit(:,t))'*eta(:,t));
    L = sum(L,2);
end

end

function RecessionPlot(dates,Recessions)
% Copyright 2010 The MathWorks, Inc.
% $Revision: 1.1.6.1 $   $Date: 2009/09/23 14:00:52 $

for r = 1:size(Recessions,1)
    
    if Recessions(r,2) > min(dates) && Recessions(r,1) <= max(dates)
        
        ph(r) = patch([Recessions(r,1) Recessions(r,1) Recessions(r,2) Recessions(r,2)], ...
            [get(gca,'YLim') fliplr(get(gca,'YLim'))], [0 0 0 0], 'k');
        set(ph(r),'FaceColor','r','FaceAlpha',0.4,'EdgeColor','none');
        
    end
    
end
end

function [residuals,Qh_pval,Qh_mod_pval] = Portmanteau_test(beta_est,omega,z,intercept,bold_y,Xbar,K,T,M,p,r,trend,xiT,DeltaY,Yminus1)

if beta_est == 'n'
    thetahat = omega(1:z);
    if intercept == 'n'
        residuals = bold_y-kron(Xbar,eye(K))*thetahat;
        residuals = reshape(residuals,K,T);
    else
        XXbar = Xbar(:,M+1:end,M);
        residuals = bold_y - kron(XXbar,eye(K))*thetahat(K*M+1:end,1);
        for m = 1:M
            residuals = residuals - kron(xiT(m,:)',eye(K))*thetahat(1+K*(m-1):K*m,1);
        end
        residuals = reshape(residuals,K,T);
    end
else
    beta = eye(r);
    if intercept == 'n'
        nu1 = omega(1:K);
        if trend == 'n'
            alpha = reshape(omega(1 + K : K + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(1+r) : K*(1+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(1+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu1 Gamma];
        else
            nu2 = omega(1 + K : 2*K);
            alpha = reshape(omega(1 + 2*K : 2*K + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(2+r) : K*(2+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(2+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu1 nu2 Gamma];
        end
        Pi = -alpha*beta';
        residuals = DeltaY - Pi*Yminus1 - Gam*Xbar';
    else
        nu1 = reshape(omega(1:K*M),K,M);
        XXbar = Xbar(:,M+1:end,M);
        if trend == 'n'
            alpha = reshape(omega(1 + K*M : K*M + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(M+r) : K*(M+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(M+r) + (K-r)*r : z),K,K*(p-1));
            Gam = Gamma;
        else
            nu2 = omega(1 + K*M : K + K*M);
            alpha = reshape(omega(1 + K*(1+M) : K*(1+M) + K*r),K,r);
            beta = [beta; reshape(omega(1 + K*(1+M+r) : K*(1+M+r) + (K-r)*r),K-r,r)];
            Gamma = reshape(omega(1 + K*(1+M+r) + (K-r)*r : z),K,K*(p-1));
            Gam = [nu2 Gamma];
        end
        Pi = -alpha*beta';
        residuals = DeltaY - Pi*Yminus1 - Gam*XXbar;
        for m = 1:M
            residuals = residuals - nu1(:,m)*xiT(m,:);
        end
    end
end

% Portmanteau test follows Ltkepohl p.169 - 171
h = 22;
C = zeros(K,K,h); C0 = 0;
for i = 1:h
    for t = i+1:T
        C(:,:,i) = C(:,:,i) + residuals(:,t)*residuals(:,t-i)'/T;
    end
end
for t = 1:T
    C0 = C0 + residuals(:,t)*residuals(:,t)'/T;
end

Qh = 0; Qh_mod = 0;
for i = 1:h
    Qh = Qh + trace(C(:,:,i)'/(C0)*C(:,:,i)/(C0))*T;
    Qh_mod = Qh_mod + trace(C(:,:,i)'/(C0)*C(:,:,i)/(C0))*T^2/(T-i);
end
Qh_pval = 1-chi2cdf(Qh,K^2*(h-p));
Qh_mod_pval = 1-chi2cdf(Qh_mod,K^2*(h-p));

end