% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [Phi,Psi,B]=svar(p,T,K,Ahat,U,long_restr)
% Estimates the SVAR parameters

% INPUTS:
% p - number of lags
% T: number of observations not taken as given
% K: number endogenous variables
% Ahat - parameter matrix from multivariate OLS
% U - (KxT) matrix of reduced form residuals
% long_restr - same as short_restr for restrictions on A(1)^-1*B

% OUTPUTS:
% Phi - (KxK) matrix, A(1)^-1
% Psi - (KxK) matrix, Phi*B
% B - (KxK) matrix of contemporaneous effects

Ph=zeros(K,K);
if p==0
    error('p must be greater than zero!');
elseif p==1
    Ph=Ph-Ahat;
elseif p>1 % Note: could also use else only
    for i=1:K:p*K
        Ph=Ph-Ahat(:,i:i+K-1);
    end
end
Ph=eye(K)+Ph; % lag polynomial, A(1) = I_K - A_1 - A_2 - ... - A_p.

Phi=inv(Ph); % Inverse of lag polynomial, A(1)^-1.

options = optimset('display','final','Algorithm', 'interior-point','MaxFunEvals', 20000, ...
    'MaxIter', 20000, 'DerivativeCheck','on','Diagnostics','off','GradObj','off','LargeScale','off');

% To pick up the 0 elements of Psi
Aeq = long_restr*kron(eye(K),Phi);
beq = zeros(size(Aeq,1),1);

B = sqrtm(U*U'/(T-K*p-1));
pars0 = B(:);
[pars,ll] = fmincon(@(pars) objfun(pars,T,K,U),pars0,[],[],Aeq,beq,[],[],[],options);
B = reshape(pars,K,K);
Psi = Phi*B;

end

function l = objfun(pars,T,K,U)

B = reshape(pars,K,K);

l = T*log(abs(det(B))) + 0.5*trace((B*B')\(U*U'));

end