% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [K,T,alpha,beta,nu1,nu2,Gamma,Xi_tilde,B,llike,llike_vecm,AIC,SC,U] = svecm(y,p,r,long_restr,short_restr,fixed,trend)
% This function first uses the method of Reduced Rank (RR) regression to
% calculate the reduced form VECM parameters and then calculates the
% structural parameters of the B-model subject to user specified restrictions.
% The model being estimated is:
% Delta y_t = Pi*y_t-1 + Psi*D_t + Gamma_1*Delta y_t-1 + ... + Gamma_p-1*Delta y_t-p+1 + u_t
% and Pi = -alpha*beta', where beta is the matrix containing the
% cointegrating relations. D_t contains deterministic terms and when
% trend = 'y', D_t = nu1 + nu2*t, otherwise D_t = nu1.

% Inputs
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables (output)
    % T: number of observations not taken as given (output)
% p: number of lags as in the VAR form of the VECM
% r: cointegration rank
% long_restr: matrix with long-run restrictions in the form of (no. of restrictions x K^2), ones pick up restricted parameters, zeros elsewhere
% short_restr: same as long_restr for restrictions directly on B
% fixed: starting values of the optimization algorithm estimating the restricted parameters, if fixed = 'y' starting values are 0.1, if = 'n' then starting values are random numbers
% trend: if trend = 'y', D_t = nu1 + nu2*t, otherwise D_t = nu1.

% Output
% alpha - (Kxr) loadings matrix
% beta - (Kxr) matrix of cointegration relationships
% nu1 - (Kx1) matrix of intercept terms
% nu2 - (Kx1) matrix of trend terms (only if trend = 'Y')
% Gamma - (KxK(p-1)) matrix of VEC parameters
% Xi_tilde - (KxK) matrix from Granger's representation theorem
% B - (KxK) matrix of contemporaneous effects
% llike: log-likelihood of optimization algorithm for the restricted parameters
% llike_vecm: log-likelihood of VEC model
% AIC: Akaike Information Criterion
% SC: Schwartz Criterion
% U - (KxT) matrix of reduced form residuals

% Make sure p is not less than 2
if p < 2
    error('p must be greater than or equal to 2')
else
%% Reduced Rank Regression to get point estimates of VECM parameters
% Procedure is according to Hamilton (1994) p.635 - 638.

% Step 1: Calculate auxiliary regressions:
% Define:
Deltay = diff(y,1,2); % Full data set, differenced
DeltaY = Deltay(:,p:end); % DY_T
Yminus1 = y(:,p:end-1); % (see also Ltkepohl (2005) p.286)
[K,T] = size(DeltaY);

% Define exogenous matrices for regressions [20.2.4] and [20.2.5], p.636 in
% Hamilton (1994). For example, with a trend term [20.2.4] is:
% Delta y_t = pi_1*const. + pi_2*trend + Pi_1*Delta y_t-1 + .... + Pi_p-1*Delta y_t-p+1 + u_t
% without a trend term the it is:
% Delta y_t = pi^hat_1*const. + Pi^hat_1*Delta y_t-1 + .... + Pi^hat_p-1*Delta y_t-p+1 + u^hat_t
if trend == 'y'
    Z = ones(K*(p-1)+2,T);
    tt = 1:T; tt = tt';
    Z(2,:)=tt;
    for i = 1:p-1
        Z(K*(i-1)+3:K*i+2,:) = Deltay(:,p-i:end-i);
    end
else
    Z = ones(K*(p-1)+1,T);
    for i = 1:p-1
        Z(K*(i-1)+2:K*i+1,:) = Deltay(:,p-i:end-i);
    end
end

% [20.2.4]
Pihat = DeltaY*Z'/(Z*Z'); % replaced inv(Z*Z') with /(Z*Z')
Uhat = DeltaY-Pihat*Z;

% [20.2.5]
Nhat = Yminus1*Z'/(Z*Z');
Vhat = Yminus1-Nhat*Z;

% Step 2: Calculate canonical correlations, [20.2.6] - [20.2.8]
SIGMAvv = Vhat*Vhat'/T;
SIGMAuu = Uhat*Uhat'/T;
SIGMAuv = Uhat*Vhat'/T;
SIGMAvu = SIGMAuv';

% Step 3: Calculate MLEs of parameters:
% Calculates the r largest eigenvalues and stores them in lambda, the
% matrix a contains the corresponding eigenvectors
[a lambda] = eigs(SIGMAvv\SIGMAvu/(SIGMAuu)*SIGMAuv,r); %#ok<NASGU>

beta_b = zeros(size(a));
for i = 1:size(a,2)
    beta_b(:,i) = a(:,i)/(a(:,i)'*SIGMAvv*a(:,i))^0.5; % [20.2.11]
end
beta_r = beta_b(1:r,:); % Normalization as in Ltkepohl (2005)
beta = beta_b/(beta_r); % the beta matrix

alpha = -SIGMAuv*beta_b*beta_r'; % the alpha matrix

Pi = -alpha*beta'; % estimated Pi matrix

% Parameter matrices:
if trend == 'y'
    for i = 1:p-1
        Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+3:K*i+2) - Pi*Nhat(:,K*(i-1)+3:K*i+2); % [20.2.13]
    end
    nu1 = Pihat(:,1) - Pi*Nhat(:,1); % constant term, [20.2.14]
    nu2 = Pihat(:,2) - Pi*Nhat(:,2); % trend term
    % Estimated reduced form covariance matrix
    Gam = [nu1 nu2 Gamma];
    U = DeltaY-Pi*Yminus1-Gam*Z;
    S_uhat = U*U'/T; % Ltkepohl (2005), p.295
    n = K*(2+K*(p-1)) + K*r + r*(K-r) + K^2 - size(short_restr,1);
else
    for i = 1:p-1
        Gamma(:,K*(i-1)+1:K*i) = Pihat(:,K*(i-1)+2:K*i+1) - Pi*Nhat(:,K*(i-1)+2:K*i+1);
    end
    nu1 = Pihat(:,1) - Pi*Nhat(:,1);
    nu2 = [];
    Gam = [nu1 Gamma];
    U = DeltaY-Pi*Yminus1-Gam*Z;
    S_uhat = U*U'/T;
    n = K*(1+K*(p-1)) + K*r + r*(K-r) + K^2 - size(short_restr,1);
end

llike_vecm = -K*T/2*log(2*pi) - T/2*log(det(S_uhat)) - 1/2*trace((DeltaY-Pi*Yminus1-Gam*Z)'/(S_uhat)*(DeltaY-Pi*Yminus1-Gam*Z));
AIC = -2*(llike_vecm-n);
SC = -2*llike_vecm+log(T)*n;
% End of reduced form estimates. 

%% Estimates of structural parameters:

% Matrix of long-run effects, Ltkepohl (2005) p. 376
beta_orth = null(beta','r');
alpha_orth = null(alpha','r');
lagpolynomial = zeros(K,K);
for i = 1:p-1
    lagpolynomial = lagpolynomial - Gamma(:,K*(i-1)+1:K*i);
end
lagpolynomial = eye(K) + lagpolynomial;
Xi_tilde = beta_orth/(alpha_orth'*lagpolynomial*beta_orth)*alpha_orth';

% To see if there are any restrictions on B:
if size(long_restr,1) + size(short_restr,1) == 0
    B = [];
    llike = [];
else
   
    % Restrictions on the B matrix specified as in (9.2.3), p.370 of Ltkepohl (2005)
    if size(long_restr,1) == 0
        C = short_restr;
    elseif size(short_restr,1) == 0
        long_restr = long_restr*kron(eye(K),Xi_tilde);
        C = long_restr;
    else
        long_restr = long_restr*kron(eye(K),Xi_tilde);
        C = [long_restr; short_restr];
    end
    
    % Optimization algorithm according to Amisano & Giannini (1997)
    % Define the restrictions
    R = null(C,'r'); % in the format of (9.1.12), picks up the unrestricted elements
    
    % Set starting values
    if fixed == 'y'
        start_val = ones(size(R,2),1)*0.1;
    else
        start_val = randn(size(R,2),1);
    end
    
    vecB = R*start_val;
    
    Com = Commutation(K);
    B = reshape(vecB,K,K);
    
    % Using (9.3.4) and (9.3.8) in (9.3.7)
    i = 0;
    while i <= 500
        
        I = T*R'*-(kron(eye(K),inv(B')))*(eye(K^2) + Com)*(-(kron(eye(K),inv(B))))*R;
        score = R'*-(kron(inv(B),inv(B')))*T*(reshape(B',K^2,1) - kron(S_uhat,eye(K))*reshape(inv(B),K^2,1));
        
        length = max(abs(I\score));
        if length > 1
            step = 1/length;
        else
            step = 1;
        end
        
        start_val = start_val + I\score*step;
        vecB = R*start_val;
        B = reshape(vecB,K,K);
        
        i = i + 1;
    end
    
    % Look at positive shocks only
    for i = 1:K
        if B(i,i) < 0
            B(:,i) = -1*B(:,i);
        end
    end
    
    % log-likelihood:
    llike = -T*log(det(B)) - T/2*trace((B*B')\S_uhat);
end % for determining whether the structural parameters should be estimated or not
end % for the if error message
end % for the function

function [Com] = Commutation(K)
% Computes the commutation matrix

Com = zeros(K^2,K);
c = 1;
r = 1;
while c <= K
    Com(r,c) = 1;
    c = c + 1;
    r = r + K;
end
zold = Com;
cc = 1;
while cc < K
    z = [zeros(cc,K); zold(1:K^2-cc,:)];
    Com = [Com z]; %#ok<AGROW>
    cc = cc + 1;
end

end