% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [Wtest] = waldtest(K,M,omega,cov)
% This function gives the Wald test reults for joint equality
% of the Lambda matrices parameters. It is generalized to states, lags and
% number of variables.
% The Wald tests take the form of l_11(1) = l_22(1), l_11(2) = l_22(2), ... ,
% l_11(M) = l_22(M). For a single state the tests would be l_11(1) =
% l_22(1), l_11(1) = l_33(1), ... , l_11(1) = l_KK(1), l_22(1) = l_33(1),
% ... , l_K-1K-1(1) = l_KK(1).

% Inputs
% K: number endogenous variables (output of svecm.m)
% M: number of Markov states
% omega - matrix with column of parameter estimates and colum of their respective standard errors (output of mssvecm.m or mssvecm_unrestricted_B.m).
% cov - covariance matrix of model parameters. Dimension depends on the type of model specified.

% Output
% Wtest - (Kx(K-1)/2x2) matrix with Wald test values and respective p-values.

parameters = omega(:,1);

%% Define the Restriction matrix
A = [ones(K-1,1) -eye(K-1,K-1)];
BB = A;

i = 2;
while i < K
    Z = [zeros(K-i,i-1) BB(1:K-i,1:K-i+1)];
    A = [A; Z]; %#ok<AGROW>
    i = i + 1;
end

Restrictions = zeros((M-1)*K*(K-1)/2,K*(M-1));
for m = 1:M-1
    Restrictions(K*(K-1)/2*(m-1)+1:K*(K-1)/2*m,K*(m-1)+1:K*m) = A;
end
Restrictions = [zeros((M-1)*K*(K-1)/2,size(omega,1)-K*(M-1)) Restrictions];

Ceq = zeros(M-1,size(Restrictions,2),K*(K-1)/2);
j = 1;
for m =1:M-1
    for i = 1:size(Ceq,3)
        Ceq(m,:,i) = Restrictions(j,:);
        j = j + 1;
    end
end

%% Obtain the Wald test results

df = M-1;
Wtest = zeros(size(Ceq,3),2);
for i = 1:size(Ceq,3)
    Wtest(i,1) = parameters'*Ceq(:,:,i)'/(Ceq(:,:,i)*cov*Ceq(:,:,i)')*Ceq(:,:,i)*parameters;
    Wtest(i,2) = 1 - chi2cdf(Wtest(i,1),df);
end

display('The Wald test and p value are')
display(Wtest)