% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [Xbar,z] = xbar(r,p,T,K,M,beta,Yminus1,Deltay,intercept,trend,beta_est)
% Determines which kind of matrix of explanatory variables to use based on user specifications:

% Inputs
% r: cointegration rank
% p: number of lags as in the VAR form of the VECM
% T: number of observations not taken as given (output of svecm.m)
% K: number endogenous variables (output of svecm.m)
% M: number of Markov states
% beta - (Kxr) matrix of cointegration relationships (output of svecm.m)
% Yminus1 - (KxT) matrix of endogenous variables in levels (see mssvecm.m)
% Deltay - (KxT) matrix of endogenous variables in first differences (see mssvecm.m)
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise. Note, there is always an intercept term in the model.
% trend: if trend = 'y', D_t = nu1 + nu2*t, otherwise D_t = nu1 (see svecm.m). Note, the trend term does not switch.
% beta_est: if beta_est = 'y' the beta matriy is estimated in each optimization round, if beta_est = 'n' the original beta matrix from svecm.m is kept throughout.

% Outputs
% Xbar - matrix containing the right hand side (RHS) exogenous terms. Dimension depends on user specification.
% z: number of parameters to be estimated on the (RHS) of the equation.

if beta_est == 'n'
    if intercept == 'n'
        if trend == 'n'
            Xbar = ones(T,1+r+K*(p-1));
            Xbar(:,2:1+r) = (beta'*Yminus1)';
            for i = 1:p-1
                Xbar(:,2+r+K*(i-1):1+r+K*i) = Deltay(:,p-i:end-i)';
            end
            z = K*(1+r+K*(p-1));
        else
            tt = 1:T; tt = tt';
            Xbar = ones(T,2+r+K*(p-1));
            Xbar(:,2) = tt;
            Xbar(:,3:2+r) = (beta'*Yminus1)';
            for i = 1:p-1
                Xbar(:,3+r+K*(i-1):2+r+K*i) = Deltay(:,p-i:end-i)';
            end
            z = K*(2+r+K*(p-1));
        end
    else % model with switching intercept
        I_M = eye(M);
        if trend == 'n'
            Xbar = zeros(T,M+r+K*(p-1),M);
            for m = 1:M
                Xbar(:,1:M,m) = kron(ones(T,1),I_M(:,m)');
                Xbar(:,M+1:M+r,m) = (beta'*Yminus1)';
                for i = 1:p-1
                    Xbar(:,M+r+1+K*(i-1):M+r+K*i,m) = Deltay(:,p-i:end-i)';
                end
            end
            z = K*(M+r+K*(p-1)); % The M*K intercept terms, K*r alpha matrix and K^2*(p-1) Gamma matrices parameters
        else
            tt = 1:T; tt = tt';
            Xbar = zeros(T,M+1+r+K*(p-1),M);
            for m = 1:M
                Xbar(:,1:M,m) = kron(ones(T,1),I_M(:,m)');
                Xbar(:,M+1,m) = tt;
                Xbar(:,M+2:M+1+r,m) = (beta'*Yminus1)';
                for i = 1:p-1
                    Xbar(:,M+2+r+K*(i-1):M+1+r+K*i,m) = Deltay(:,p-i:end-i)';
                end
            end
            z = K*(M+1+r+K*(p-1)); % The M*K intercept terms, K trend terms, K*r alpha matrix and K^2*(p-1) Gamma matrices parameters
        end
    end
else
    if intercept == 'n'
        if trend == 'n'
            Xbar = ones(T,1+K*(p-1));
            for i = 1:p-1
                Xbar(:,2+K*(i-1):1+K*i) = Deltay(:,p-i:end-i)';
            end
            z = K*(1+K*(p-1)+r) + (K-r)*r;
        else
            tt = 1:T; tt = tt';
            Xbar = ones(T,2+K*(p-1));
            Xbar(:,2) = tt;
            for i = 1:p-1
                Xbar(:,3+K*(i-1):2+K*i) = Deltay(:,p-i:end-i)';
            end
            z = K*(2+K*(p-1)+r) + (K-r)*r;
        end
    else % model with switching intercept
        I_M = eye(M);
        if trend == 'n'
            Xbar = zeros(T,M+K*(p-1),M);
            for m = 1:M
                Xbar(:,1:M,m) = kron(ones(T,1),I_M(:,m)');
                for i = 1:p-1
                    Xbar(:,M+1+K*(i-1):M+K*i,m) = Deltay(:,p-i:end-i)';
                end
            end
            z = K*(M+K*(p-1)+r) + (K-r)*r;
        else
            tt = 1:T; tt = tt';
            Xbar = zeros(T,M+1+K*(p-1),M);
            for m = 1:M
                Xbar(:,1:M,m) = kron(ones(T,1),I_M(:,m)');
                Xbar(:,M+1,m) = tt;
                for i = 1:p-1
                    Xbar(:,M+2+K*(i-1):M+1+K*i,m) = Deltay(:,p-i:end-i)';
                end
            end
            z = K*(M+1+K*(p-1)+r) + (K-r)*r;
        end
    end
end